#####################################################-*- makefile -*-##
# CERT SiLK Analysis Suite Global Makefile
# $SiLK: build.mk 4524 2006-08-17 19:02:40Z mwd $
#######################################################################

include $(top_srcdir)/target.mk

# option to make used when calling onesself recursively without
# changing directories.
MAKE_RECURSIVE_OPT += -f $(top_srcdir)/recurse.mk


########################################################################
## Dependency generation in the Makefile
########################################################################
#
## Temporary makefile
#MAKEFILE_MAKEDEPEND=Makefile.MAKEDEPEND
#
## Generate a list of all object files for all targets in the Makefile;
## we use sort here to eliminate duplicates.
#ALL_OBJS=$(sort $(LIBOBJS) $(OBJS) $(TESTOBJS) $(DLOBJS) $(foreach lib,$(LIBRARIES),$(LIBOBJS_$(lib))) $(foreach app,$(APPNAMES),$(OBJS_$(app))) $(foreach tapp,$(TESTAPPS),$(TESTOBJS_$(tapp))) $(foreach dl,$(DYNALIBS),$(DLOBJS_$(dl))))
#
## Get any target-specific CFLAGS settings from the local makefile
#CFLAGS_TARGET=$(foreach tgt,$(APPNAMES) $(LIBRARIES) $(TESTAPPS) $(DYNALIBS),$(CFLAGS_$(tgt)))
#
## Create a fake depency target for each object file
#ALL_DEPS=$(addprefix MAKEDEPEND-,$(ALL_OBJS))
#
## How to generate the dependencies for each object file
#$(ALL_DEPS): MAKEDEPEND-%.o: %.c
#	$(CC) $(CFLAGS_MAKEDEPEND) $(CFLAGS_TARGET) $(CFLAGS_LOCAL) $(CFLAGS_COMMON) $^ | sed 's,\($*\.o[ :]*\),$(O)\1,g' >>Makefile.MAKEDEPEND
#
## Copy the Makefile up to the start of the dependencies
#$(MAKEFILE_MAKEDEPEND): Makefile always_rebuild
#	sed '/# DO NOT DELETE THIS LINE -- make depend depends/q' < $< > $@
#
## The target; depends on the start of the Makefile plus dependency for
## each object listed in the Makefile
#depend: $(MAKEFILE_MAKEDEPEND) $(ALL_DEPS)
#	$(MV) $< Makefile
#
#.PHONY: $(MAKEFILE_MAKEDEPEND) $(ALL_DEPS)
#
#
#######################################################################
# Dependent library recursive make rules
#######################################################################

ifndef NO_CHECK_DEPLIBS

# Generate list of all SiLK libraries required by the targets in this
# directory, though not all these libraries may be required by target
# we are building.
ALL_DEPLIBS=$(sort $(DEPLIBS) $(foreach tgt,$(APPNAMES) $(TESTAPPS) $(DYNALIBS),$(DEPLIBS_$(tgt))))

$(ALL_DEPLIBS): %: always_rebuild
	@echo Checking whether $(@F) needs rebuilding
	$(MAKE) -C $(top_builddir)/src/$(@D) libraries LIBRARIES=$(@F:lib%.a=%)

endif

check-deplibs: $(ALL_DEPLIBS)


#######################################################################
# Functions to be invoked by $(call xxx,args)
#######################################################################

# $(call mkobjs,NAMES)
#     For each name in NAMES, this function prepends "$(O)" to the
#     name.
# Ex: $(call mkobjs,foo.o bar.o baz.o)
#         ==> $(O)foo.o $(O)bar.o $(O)baz.o

mkobjs = $(foreach name,$(1),$(O)$(name))


# $(call mklibs,DEPLIBS)
#     For each lib in DEPLIBS, where lib looks like "libsilk/libsilk.a",
#     this function prepends "$(top_builddir)/src" to the lib, and
#     inserts "$(O)" just between the lib's dirname and basename.
# Ex: $(call mklibs,libsilk/libsilk.a libhash/libhash.a)
#         ==> $(top_builddir)/src/libsilk/$(O)libsilk.a $(top_builddir)/src/libhash/$(O)libhash.a)

mklibs = $(foreach lib,$(1),$(top_builddir)/src/$(dir $(lib))$(O)$(basename $(notdir $(lib))).a)


#######################################################################
# First pass through makefile, recurse with the correct object list
# for the target
#######################################################################

ifneq ($(strip $(OBJS) $(foreach app,$(APPNAMES),$(OBJS_$(app)))),)
$(APPNAMES_LOCAL): %: always_rebuild check-deplibs
	@WITHDEPS_OBJ="$(call mkobjs,$(OBJS) $(OBJS_$(*)))" WITHDEPS_LIB="$(call mklibs,$(DEPLIBS_$(*)) $(DEPLIBS))" WITHDEP_LDFLAGS="$(LDFLAGS_$(*))" WITHDEP_CFLAGS="$(CFLAGS_$(*))" APPNAMES="$@" $(MAKE) $(MAKE_RECURSIVE_OPT) $@
endif


ifneq ($(strip $(TESTOBJS) $(foreach app, $(TESTAPPS), $(TESTOBJS_$(app)))),)
$(TESTAPPS_LOCAL): %: always_rebuild check-deplibs
	@WITHDEPS_OBJ="$(call mkobjs,$(TESTOBJS) $(TESTOBJS_$(*)))" WITHDEPS_LIB="$(call mklibs,$(DEPLIBS_$(*)) $(DEPLIBS))" WITHDEP_LDFLAGS="$(LDFLAGS_$(*))" WITHDEP_CFLAGS="$(CFLAGS_$(*))" TESTAPPS="$@" $(MAKE) $(MAKE_RECURSIVE_OPT) $@
endif


ifneq ($(strip $(LIBOBJS) $(foreach lib, $(LIBRARIES), $(LIBOBJS_$(lib)))),)
$(LIBRARIES_LOCAL): $(O)lib%.a: always_rebuild
	@WITHDEPS_OBJ="$(call mkobjs,$(LIBOBJS) $(LIBOBJS_$(*)))" WITHDEP_CFLAGS="$(CFLAGS_$(*))" LIBRARIES="$(*)" $(MAKE) $(MAKE_RECURSIVE_OPT) $@
endif


ifneq ($(strip $(DLOBJS) $(foreach lib, $(DYNALIBS), $(DLOBJS_$(lib)))),)
$(DYNALIBS_LOCAL): $(O)lib%.$(SHLIB_EXT): always_rebuild check-deplibs
	@WITHDEPS_OBJ="$(call mkobjs,$(DLOBJS) $(DLOBJS_$(*)))" WITHDEPS_LIB="$(call mklibs,$(DEPLIBS_$(*)) $(DEPLIBS))" WITHDEP_LDFLAGS="$(LDFLAGS_$(*))" WITHDEP_CFLAGS="$(CFLAGS_$(*)) $(SHLIB_CCFLAGS)" DYNALIBS="$(*)" $(MAKE) $(MAKE_RECURSIVE_OPT) $@
endif
