/*
** Copyright (C) 2006 by Carnegie Mellon University.
**
** @OPENSOURCE_HEADER_START@
**
** Use of the SILK system and related source code is subject to the terms
** of the following licenses:
**
** GNU Public License (GPL) Rights pursuant to Version 2, June 1991
** Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
**
** NO WARRANTY
**
** ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER
** PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY
** PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN
** "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY
** KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT
** LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE,
** MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE
** OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT,
** SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY
** TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF
** WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES.
** LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF
** CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON
** CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE
** DELIVERABLES UNDER THIS LICENSE.
**
** Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie
** Mellon University, its trustees, officers, employees, and agents from
** all claims or demands made against them (and any related losses,
** expenses, or attorney's fees) arising out of, or relating to Licensee's
** and/or its sub licensees' negligent use or willful misuse of or
** negligent conduct or willful misconduct regarding the Software,
** facilities, or other rights or assistance granted by Carnegie Mellon
** University under this License, including, but not limited to, any
** claims of product liability, personal injury, death, damage to
** property, or violation of any laws or regulations.
**
** Carnegie Mellon University Software Engineering Institute authored
** documents are sponsored by the U.S. Department of Defense under
** Contract F19628-00-C-0003. Carnegie Mellon University retains
** copyrights in all material produced under this contract. The U.S.
** Government retains a non-exclusive, royalty-free license to publish or
** reproduce these documents, or allow others to do so, for U.S.
** Government purposes only pursuant to the copyright license under the
** contract clause at 252.227.7013.
**
** @OPENSOURCE_HEADER_END@
*/

/*
**  Support file for probeconf.c
**
*/

#define DECLARE_SENSORID_VARIABLES   1
#define DECLARE_SITE_VARIABLES       1
#include "silk.h"

RCSIDENT("$SiLK: probeconf-twoway.c 5910 2006-12-11 19:44:49Z mthomas $");

#include "probeconf_priv.h"

/* return true if the pointer to the SiLK Flow record r represents an
 * ICMP flow */
#define IS_ICMP(r)  (1 == (r)->proto)


/*
 *  Verify probe by its class.
 */
int skProbeVerifyClass(probe_def_t *probe)
{
    /* There is a single class, so no per-class verification is
     * necessary. */

    /* Make certain the probe's type is valid */
    switch (probe->probe_type) {
      case PROBE_ENUM_NETFLOW:
      case PROBE_ENUM_YAF:
        /* expected probe types */
        break;

      default:
        assert(skProbetypeEnumtoName(probe->probe_type));
        skAppPrintErr(("Cannot verify probe %s:\n"
                       "\t'%s' probes are not supported at this site."),
                      probe->probe_uniq_name,
                      skProbetypeEnumtoName(probe->probe_type));
        return -1;
    }

    /* Check for a list of IP-blocks.  If only one of "internal" or
     * "external" is given, set the other one to the remainder unless
     * the "null" ipblock was already set to remainder. */
    if (probe->if_ipblock[IFMAP_EXTERNAL] != NULL) {
        if ((probe->if_ipblock[IFMAP_INTERNAL] == NULL)
            && (probe->if_remaining_ips != IFMAP_NULL))
        {
            if (skProbeSetIpBlockRemainingIps(probe, IFMAP_INTERNAL)) {
                return -1;
            }
        }
        return 0;
    }
    if (probe->if_ipblock[IFMAP_INTERNAL] != NULL) {
        if ((probe->if_ipblock[IFMAP_EXTERNAL] == NULL)
            && (probe->if_remaining_ips != IFMAP_NULL))
        {
            if (skProbeSetIpBlockRemainingIps(probe, IFMAP_EXTERNAL)) {
                return -1;
            }
        }
        return 0;
    }

    /* we get here when neither "internal" nor "external" ipblocks
     * were specified. */

    /* YAF probes only support IPblocks, so it's an error if no
     * ipblocks were specified. */
    if (probe->probe_type == PROBE_ENUM_YAF) {
        skAppPrintErr(("Cannot verify probe %s:\n"
                       "\tThe %s-ipblock or %s-ipblock must be given"),
                      probe->probe_uniq_name,
                      ifmap_group_id_name[IFMAP_EXTERNAL],
                      ifmap_group_id_name[IFMAP_INTERNAL]);
        return -1;
    }

    /* Only NETFLOW probes should make it here */
    assert(probe->probe_type == PROBE_ENUM_NETFLOW);

    /* Although the skProbeVerify() method will call
     * skProbeVerifyNetflow(), explicitly call it now to set the
     * IFMAP_NULL value before potentially setting the "remainder"
     * below. */
    if (skProbeVerifyNetflow(probe)) {
        return -1;
    }

    /* Check for a list of SNMP interfaces.  If only one of internal
     * or external is given, set the other one to the remainder. */
    if (skBitmapGetHighCount(probe->if_map[IFMAP_EXTERNAL]) != 0) {
        if (skBitmapGetHighCount(probe->if_map[IFMAP_INTERNAL]) == 0) {
            if (skProbeSetInterfaceToRemainder(probe, IFMAP_INTERNAL)) {
                return -1;
            }
        }
        return 0;
    }
    if (skBitmapGetHighCount(probe->if_map[IFMAP_INTERNAL]) != 0) {
        if (skBitmapGetHighCount(probe->if_map[IFMAP_EXTERNAL]) == 0) {
            if (skProbeSetInterfaceToRemainder(probe, IFMAP_EXTERNAL)) {
                return -1;
            }
        }
        return 0;
    }

    /* It is an error when neither SNMP interfaces nor IP-blocks were
     * specified. */
    skAppPrintErr(("Cannot verify probe %s:\n"
                   "\tAt least one of %s-interface and %s-interface\n"
                   "\tor %s-ipblock and %s-ipblock must be given"),
                  probe->probe_uniq_name,
                  ifmap_group_id_name[IFMAP_EXTERNAL],
                  ifmap_group_id_name[IFMAP_INTERNAL],
                  ifmap_group_id_name[IFMAP_EXTERNAL],
                  ifmap_group_id_name[IFMAP_INTERNAL]);
    return -1;
}


/*
 *  count = skProbeDetermineFlowtype(probe, &rwrec, ftypes[], sensorids[]);
 *
 *    Fill the 'ftypes' and 'sensorids' arrays with the list of
 *    flow_types and sensors to which the 'rwrec' probe, collected
 *    from the 'probe' sensor, should be packed.  Return the number of
 *    elements added to each array or -1 on error.
 */
int skProbeDetermineFlowtype(
    const probe_def_t  *probe,
    const rwRec        *rwrec,
    uint8_t            *ftypes,
    sensorID_t         *sensorids)
{
    assert(ftypes);
    assert(sensorids);

    if (probe->probe_type == PROBE_ENUM_SILK) {
        /* For a record from a SiLK Flow file, return whatever
         * flowtype and sensor is on the record itself. */
        ftypes[0] = rwrec->flow_type;
        sensorids[0] = rwrec->sID;
        return 1;
    }

    /* sensor_id is always the sensor where the flow was collected */
    sensorids[0] = probe->sensor_id;

    if (skProbeTestFlowInterfaces(probe, rwrec, IFMAP_EXTERNAL, REC_DIR_FROM)
        == 1)
    {
        /* Flow reached the monitoring point from the outside */
        if (skProbeTestFlowInterfaces(probe, rwrec, IFMAP_NULL, REC_DIR_TO)
            == 1)
        {
            /* Flow went to the null destination */
            ftypes[0] = RW_IN_NULL;
            return 1;
        }
        if (skProbeTestFlowInterfaces(probe, rwrec, IFMAP_INTERNAL, REC_DIR_TO)
            == 1)
        {
            /* Flow entered the monitored network */
#ifdef SK_ENABLE_ICMP_SPLIT
            if (IS_ICMP(rwrec)) {
                ftypes[0] = RW_IN_ICMP;
                return 1;
            }
#endif
#ifdef SK_ENABLE_WEB_SPLIT
            if (IS_WEB(rwrec)) {
                ftypes[0] = RW_IN_WEB;
                return 1;
            }
#endif
            ftypes[0] = RW_IN;
            return 1;
        }
        if (skProbeTestFlowInterfaces(probe, rwrec, IFMAP_EXTERNAL, REC_DIR_TO)
            == 1)
        {
            /* Flow went back out the way it came in */
            ftypes[0] = RW_EXT2EXT;
            return 1;
        }
        /* Flow left the monitor through an unknown interface */
        ftypes[0] = RW_OTHER;
        return 1;
    }

    if (skProbeTestFlowInterfaces(probe, rwrec, IFMAP_INTERNAL, REC_DIR_FROM)
        == 1)
    {
        /* Flow reached the monitoring point from the inside of network */
        if (skProbeTestFlowInterfaces(probe, rwrec, IFMAP_NULL, REC_DIR_TO)
            == 1)
        {
            /* Flow went to the null destination */
            ftypes[0] = RW_OUT_NULL;
            return 1;
        }
        if (skProbeTestFlowInterfaces(probe, rwrec, IFMAP_EXTERNAL, REC_DIR_TO)
            == 1)
        {
            /* Flow left the monitored network */
#ifdef SK_ENABLE_ICMP_SPLIT
            if (IS_ICMP(rwrec)) {
                ftypes[0] = RW_OUT_ICMP;
                return 1;
            }
#endif
#ifdef SK_ENABLE_WEB_SPLIT
            if (IS_WEB(rwrec)) {
                ftypes[0] = RW_OUT_WEB;
                return 1;
            }
#endif
            ftypes[0] = RW_OUT;
            return 1;
        }
        if (skProbeTestFlowInterfaces(probe, rwrec, IFMAP_INTERNAL, REC_DIR_TO)
            == 1)
        {
            /* Flow went back into the monitored network */
            ftypes[0] = RW_INT2INT;
            return 1;
        }
        /* Flow went to an unknown interface */
        ftypes[0] = RW_OTHER;
        return 1;
    }

    /* Flow originated from an unknown interface */
    ftypes[0] = RW_OTHER;
    return 1;
}
