/*
** Copyright (C) 2006 by Carnegie Mellon University.
**
** @OPENSOURCE_HEADER_START@
**
** Use of the SILK system and related source code is subject to the terms
** of the following licenses:
**
** GNU Public License (GPL) Rights pursuant to Version 2, June 1991
** Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
**
** NO WARRANTY
**
** ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER
** PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY
** PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN
** "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY
** KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT
** LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE,
** MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE
** OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT,
** SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY
** TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF
** WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES.
** LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF
** CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON
** CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE
** DELIVERABLES UNDER THIS LICENSE.
**
** Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie
** Mellon University, its trustees, officers, employees, and agents from
** all claims or demands made against them (and any related losses,
** expenses, or attorney's fees) arising out of, or relating to Licensee's
** and/or its sub licensees' negligent use or willful misuse of or
** negligent conduct or willful misconduct regarding the Software,
** facilities, or other rights or assistance granted by Carnegie Mellon
** University under this License, including, but not limited to, any
** claims of product liability, personal injury, death, damage to
** property, or violation of any laws or regulations.
**
** Carnegie Mellon University Software Engineering Institute authored
** documents are sponsored by the U.S. Department of Defense under
** Contract F19628-00-C-0003. Carnegie Mellon University retains
** copyrights in all material produced under this contract. The U.S.
** Government retains a non-exclusive, royalty-free license to publish or
** reproduce these documents, or allow others to do so, for U.S.
** Government purposes only pursuant to the copyright license under the
** contract clause at 252.227.7013.
**
** @OPENSOURCE_HEADER_END@
*/

#if !defined(_SILK_SITE_H) || defined(_SILK_SITE_TWOWAY_H)
#error "Never include silk_site_twoway.h directly, include silk_site.h instead"
#endif
#define _SILK_SITE_TWOWAY_H

RCSIDENTVAR(rcsID_SILK_SITE_TWOWAY_H, "$SiLK: silk_site_twoway.h 5910 2006-12-11 19:44:49Z mthomas $");


/*
 *  Whether to split the web data separately from the other data; that
 *  is, whether to use the "inweb" and "outweb" flowtypes.
 *
 *  Web data is any flow seen on ports 80/tcp, 8080/tcp, 443/tcp.
 */
#define SK_ENABLE_WEB_SPLIT   1

/*
 *  Whether to split ICMP data separately from the other data; that
 *  is, whether to use the "inicmp" and "outicmp" flowtypes.
 *
 *  ICMP data is any flow where proto == 1.
 */
#define SK_ENABLE_ICMP_SPLIT  0


#ifdef DECLARE_SITE_VARIABLES
/*
 *  Indexes into the outFInfo array.
 *
 *    DO NOT rearrange these values, add values to the middle of this
 *    list, or remove values as they are used in rwfilter output files
 *    to record where data was collected.
 */
typedef enum {
    RW_IN,
    RW_OUT,
    RW_IN_WEB,
    RW_OUT_WEB,
    RW_IN_NULL,
    RW_OUT_NULL,
    RW_INT2INT,
    RW_EXT2EXT,
    RW_IN_ICMP,
    RW_OUT_ICMP,
    RW_OTHER
} flowtype_t;
#define NUM_FLOW_TYPES 11


#ifdef DEFINE_SITE_VARIABLES

#ifdef SK_ENABLE_INITIAL_TCPFLAGS
#  define FILE_FORMAT_NONWEB FT_RWAUGMENTED
#  define FILE_FORMAT_WEB    FT_RWAUGWEB
#else
#  define FILE_FORMAT_NONWEB FT_RWSPLIT
#  define FILE_FORMAT_WEB    FT_RWWWW
#endif

/*
**  The following macro is used to fill in the outFInfo[] array below.
**  This macro fills in the fileTypeInfo struct, defined in rwpack.h
**  and repeated here:
**
**  className, dirPrefix, filePrefix, pathPrefix, packType
 */
#define OUT_F_INFO_ENTRY(dirPrefix,filePrefix,packType)                 \
    {"all", (dirPrefix), (filePrefix), (dirPrefix "/"), (packType)}

fileTypeInfo outFInfo[NUM_FLOW_TYPES] = {
    /* dirPrefix, filePrefilx, packType */
    OUT_F_INFO_ENTRY("in",       "in",       FILE_FORMAT_NONWEB),
    OUT_F_INFO_ENTRY("out",      "out",      FILE_FORMAT_NONWEB),
    OUT_F_INFO_ENTRY("inweb",    "iw",       FILE_FORMAT_WEB),
    OUT_F_INFO_ENTRY("outweb",   "ow",       FILE_FORMAT_WEB),
    OUT_F_INFO_ENTRY("innull",   "innull",   FILE_FORMAT_NONWEB),
    OUT_F_INFO_ENTRY("outnull",  "outnull",  FILE_FORMAT_NONWEB),
    OUT_F_INFO_ENTRY("int2int",  "int2int",  FILE_FORMAT_NONWEB),
    OUT_F_INFO_ENTRY("ext2ext",  "ext2ext",  FILE_FORMAT_NONWEB),
    OUT_F_INFO_ENTRY("inicmp",   "inicmp",   FILE_FORMAT_NONWEB),
    OUT_F_INFO_ENTRY("outicmp",  "outicmp",  FILE_FORMAT_NONWEB),
    OUT_F_INFO_ENTRY("other",    "other",    FILE_FORMAT_NONWEB)
};

/* compute number of file types */
uint32_t numFileTypes = sizeof(outFInfo)/sizeof(fileTypeInfo);
#endif /* DEFINE_SITE_VARIABLES */
#endif /* DECLARE_SITE_VARIABLES */


/************** Classes */

#ifdef DECLARE_CLASSINFO_VARIABLES
#define NUM_CLASSES 1

#define DEFAULT_CLASS 0

extern classInfoStruct classInfo[];

#ifdef DEFINE_CLASSINFO_VARIABLES
/*
 * Types associated with classes
 *    {"className", NUM_TYPES (number from 1 to 12), NUM_DEFAULTS (1 to 4),
 *     {TYPE0,TYPE1,...,TYPE11} (types: array[12] of flowtype_t)
 *     {DEF_TYPE0,...,DEF_TYPE3} (default types: array[4] of flowtype_t)}
 */
classInfoStruct classInfo[NUM_CLASSES] = {
    {"all", NUM_FLOW_TYPES, 2,
     {RW_IN, RW_OUT, RW_IN_WEB, RW_OUT_WEB, RW_IN_NULL, RW_OUT_NULL,
      RW_INT2INT, RW_EXT2EXT, RW_IN_ICMP, RW_OUT_ICMP, RW_OTHER, 0},
     {RW_IN, RW_IN_WEB, RW_IN_ICMP, 0}}, /* 0 */
};
#endif /* DEFINE_CLASSINFO_VARIABLES */
#endif /* DECLARE_CLASSINFO_VARIABLES */


#include "sensors_twoway.h"

/*
** Local variables:
** mode:c
** indent-tabs-mode:nil
** c-basic-offset:4
** End:
*/
