/*
** Copyright (C) 2004-2006 by Carnegie Mellon University.
**
** @OPENSOURCE_HEADER_START@
** 
** Use of the SILK system and related source code is subject to the terms 
** of the following licenses:
** 
** GNU Public License (GPL) Rights pursuant to Version 2, June 1991
** Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
** 
** NO WARRANTY
** 
** ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER 
** PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY 
** PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN 
** "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY 
** KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT 
** LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE, 
** MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE 
** OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT, 
** SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY 
** TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF 
** WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES. 
** LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF 
** CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON 
** CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE 
** DELIVERABLES UNDER THIS LICENSE.
** 
** Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie 
** Mellon University, its trustees, officers, employees, and agents from 
** all claims or demands made against them (and any related losses, 
** expenses, or attorney's fees) arising out of, or relating to Licensee's 
** and/or its sub licensees' negligent use or willful misuse of or 
** negligent conduct or willful misconduct regarding the Software, 
** facilities, or other rights or assistance granted by Carnegie Mellon 
** University under this License, including, but not limited to, any 
** claims of product liability, personal injury, death, damage to 
** property, or violation of any laws or regulations.
** 
** Carnegie Mellon University Software Engineering Institute authored 
** documents are sponsored by the U.S. Department of Defense under 
** Contract F19628-00-C-0003. Carnegie Mellon University retains 
** copyrights in all material produced under this contract. The U.S. 
** Government retains a non-exclusive, royalty-free license to publish or 
** reproduce these documents, or allow others to do so, for U.S. 
** Government purposes only pursuant to the copyright license under the 
** contract clause at 252.227.7013.
** 
** @OPENSOURCE_HEADER_END@
*/
#ifndef _FCINTERNALS_H
#define _FCINTERNALS_H

#include "silk.h"

RCSIDENTVAR(rcsID_FCINTERNALS_H, "$SiLK: fcinternals.h 5040 2006-09-26 19:38:02Z mwd $");

/*
**  fcinternals.h
**
**  Common information between flowcap objects
**
**/

#include "skdeque.h"
#include "lzo-file.h"
#include "sktimer.h"
#include "pdusource.h"
#ifdef HAVE_YAF_YAFCORE_H
#include "yafsource.h"
#endif
#include "probeconf.h"

typedef void (*cleanupHandler)(void *); /* Type of cleanup handlers */

typedef struct fileStruct {
    struct fileStruct *next;    /* next in queue */
    uint32_t startTime;         /* epoch at start */
    uint32_t endTime;           /* epoch at end of writing */
    uint32_t sendStartTime;     /* epoch at beginning of sending */
    uint32_t sentTime;          /* epoch at end of sending */
    uint8_t  isRamDisk;         /* 1 if ram disk file */
    uint8_t  sendingTries; /* number of times we tried to send file */
    uint64_t size;              /* Size of file */
    uint64_t uncompressSize;    /* Size if uncompressed */
    char     fName[56];         /* file basename */
} fileStruct;
typedef fileStruct *fileStructPtr;

typedef struct writeFileStruct {
    uint32_t type;              /* type */
    uint32_t HWM;               /* High Water Mark: send limit */
    lzo_compr_buffer_t lzofile; /* lzofile abstraction */
    pthread_mutex_t mutex;      /* mutex */
    const probe_def_t *probe;

    pthread_t reader_thread;
    union {
        pduSource_t pdu;
#ifdef HAVE_YAF_YAFCORE_H
        yafSource_t yaf;
#endif
    } source;

    fileStructPtr fPtr;         /* which file we are writing to*/
    uint32_t bufLen;            /* where to write in buffer next */
    skTimer_t timerHandle;      /* send timer handle */
    FILE *gzFILE;               /* gzfile pointer */
    uint32_t records;           /* Number of records */
    uint32_t headerSize;        /* Size of the header */
    uint8_t version;            /* Version of flowcap to write */
    unsigned close : 1;   /* Whether this file is due to be closed. */
    unsigned closing : 1;       /* Whether this file is in the process
                                   of being closed. */
    unsigned valid_source : 1;
    unsigned running : 1;    
} writeFileStruct;
typedef writeFileStruct *writeFileStructPtr;

/* imported functions and variables */
void closeWFile(writeFileStructPtr wfPtr, uint8_t open);

extern volatile uint8_t reading; /* Reader shut down flag (0 == stop) */

typedef struct fileQueue fileQueue;

uint32_t dumpHeader(writeFileStructPtr wfPtr);

void *reader(void *);

#endif /* _FCINTERNALS_H */

/*
** Local variables:
** mode:c
** indent-tabs-mode:nil
** c-basic-offset:4
** End:
*/
