/*
** Copyright (C) 2001-2006 by Carnegie Mellon University.
**
** @OPENSOURCE_HEADER_START@
** 
** Use of the SILK system and related source code is subject to the terms 
** of the following licenses:
** 
** GNU Public License (GPL) Rights pursuant to Version 2, June 1991
** Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
** 
** NO WARRANTY
** 
** ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER 
** PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY 
** PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN 
** "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY 
** KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT 
** LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE, 
** MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE 
** OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT, 
** SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY 
** TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF 
** WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES. 
** LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF 
** CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON 
** CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE 
** DELIVERABLES UNDER THIS LICENSE.
** 
** Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie 
** Mellon University, its trustees, officers, employees, and agents from 
** all claims or demands made against them (and any related losses, 
** expenses, or attorney's fees) arising out of, or relating to Licensee's 
** and/or its sub licensees' negligent use or willful misuse of or 
** negligent conduct or willful misconduct regarding the Software, 
** facilities, or other rights or assistance granted by Carnegie Mellon 
** University under this License, including, but not limited to, any 
** claims of product liability, personal injury, death, damage to 
** property, or violation of any laws or regulations.
** 
** Carnegie Mellon University Software Engineering Institute authored 
** documents are sponsored by the U.S. Department of Defense under 
** Contract F19628-00-C-0003. Carnegie Mellon University retains 
** copyrights in all material produced under this contract. The U.S. 
** Government retains a non-exclusive, royalty-free license to publish or 
** reproduce these documents, or allow others to do so, for U.S. 
** Government purposes only pursuant to the copyright license under the 
** contract clause at 252.227.7013.
** 
** @OPENSOURCE_HEADER_END@
*/
#ifndef _SILK_SITE_H
#define _SILK_SITE_H

/* FIXME. Find a good home the structures declared in this file */

#include "silk.h"

RCSIDENTVAR(rcsID_SILK_SITE_H, "$SiLK: silk_site.h 4301 2006-07-19 21:50:10Z mthomas $");


/* Find a home for these constants and structures definitions */

#ifdef DECLARE_SITE_VARIABLES

/* FIXME. Ultimately, we'll want to split out the static type
 * definition information from the rwpack state, and move the static
 * configuration information to another file. */
typedef struct fileTypeInfo {
    /* The 'class' the data in this file belongs to */
    const char *className;
    /* The 'type' the data in this file belongs to. */
    const char *dirPrefix;
    /* The flowtype the data in this file belongs to. */
    const char *filePrefix;
    /* Combination of the className and dirPrefix. */
    const char *pathPrefix;
    /* The file output format to use: one of FT_RW* from silk_files.h */    
    int32_t     packType;
} fileTypeInfo;


/* A site-specific "implementation header", silk_site_*.h declares the
 * following statically populated deployment-specific global data
 * structures. */
extern fileTypeInfo outFInfo[];
extern uint32_t numFileTypes;



#endif /* DECLARE_SITE_VARIABLES */



#ifdef DECLARE_CLASSINFO_VARIABLES

#define MAX_TYPES_PER_CLASS     12
#define FGLOB_MAX_DEFAULT_TYPES  4

typedef struct classInfoStruct {
    char *className;
    uint8_t numTypes;
    uint8_t numDefaults;
    uint8_t typeList[MAX_TYPES_PER_CLASS];
    uint8_t defaultList[FGLOB_MAX_DEFAULT_TYPES];
} classInfoStruct;

#endif /* DECLARE_CLASSINFO_VARIABLES */



#ifdef DECLARE_SENSORID_VARIABLES

#define MAX_CLASSES_PER_SENSOR   8

typedef struct {
    char *sensorName;
    uint8_t numClasses;
    uint8_t classNumbers[MAX_CLASSES_PER_SENSOR];
} sensorinfo_t;

extern uint32_t numSensors;
extern sensorinfo_t sensorInfo[];

/* Put this as the last entry in the sensorInfo[] array */
#define SENSOR_INFO_SENTINEL  {NULL, 0, {0, 0, 0, 0, 0, 0, 0, 0}}

#endif /* DECLARE_SENSORID_VARIABLES */



/* Include the site-specific header; this macro is set in silk_conifg.h */
#include SILK_SITE_HEADER_NAME


extern const char *ifmap_group_id_name[];

/* Set the default SNMP interface groupings if silk_site_*.h did not. */
#ifndef NUM_IFMAPS

/* Number of enums defined below */
#define NUM_IFMAPS      3

typedef enum {
    /* A bitmap where an ON bit represents an SNMP interface on the
     * probe that faces the external world.  Traffic entering the
     * router on this interface will be considered incoming. */
    IFMAP_EXTERNAL,

    /* A bitmap where an ON bit represents an SNMP interface on the
     * probe that faces the internal world.  Traffic entering the
     * router on this interface will be considered outgoing. */
    IFMAP_INTERNAL,

    /* The SNMP interface on the probe to which non-routed traffic is
     * sent.  This is 0 on Cisco routers. */
    IFMAP_NULL
} ifmap_group_id_t;

#ifdef DECLARE_IFMAP_VARIABLES
/* names for the above.  Keep in sync!! */
const char *ifmap_group_id_name[NUM_IFMAPS] = {
    "external",
    "internal",
    "null"
};
#endif

#endif /* !defined(NUM_IFMAPS) */

#endif /* _SILK_SITE_H */
