/*
** Copyright (C) 2004-2006 by Carnegie Mellon University.
**
** @OPENSOURCE_HEADER_START@
** 
** Use of the SILK system and related source code is subject to the terms 
** of the following licenses:
** 
** GNU Public License (GPL) Rights pursuant to Version 2, June 1991
** Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
** 
** NO WARRANTY
** 
** ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER 
** PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY 
** PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN 
** "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY 
** KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT 
** LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE, 
** MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE 
** OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT, 
** SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY 
** TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF 
** WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES. 
** LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF 
** CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON 
** CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE 
** DELIVERABLES UNDER THIS LICENSE.
** 
** Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie 
** Mellon University, its trustees, officers, employees, and agents from 
** all claims or demands made against them (and any related losses, 
** expenses, or attorney's fees) arising out of, or relating to Licensee's 
** and/or its sub licensees' negligent use or willful misuse of or 
** negligent conduct or willful misconduct regarding the Software, 
** facilities, or other rights or assistance granted by Carnegie Mellon 
** University under this License, including, but not limited to, any 
** claims of product liability, personal injury, death, damage to 
** property, or violation of any laws or regulations.
** 
** Carnegie Mellon University Software Engineering Institute authored 
** documents are sponsored by the U.S. Department of Defense under 
** Contract F19628-00-C-0003. Carnegie Mellon University retains 
** copyrights in all material produced under this contract. The U.S. 
** Government retains a non-exclusive, royalty-free license to publish or 
** reproduce these documents, or allow others to do so, for U.S. 
** Government purposes only pursuant to the copyright license under the 
** contract clause at 252.227.7013.
** 
** @OPENSOURCE_HEADER_END@
*/
#ifndef _PREFIXTREE_H
#define _PREFIXTREE_H

#include "silk.h"

RCSIDENTVAR(rcsID_PREFIXTREE_H, "$SiLK: skprefixmap.h 4877 2006-09-14 21:19:18Z mthomas $");

/*
** prefixmap.h
** 
** John McClary Prevost
** December 1st, 2004
**
** This is a tree data structure for prefixes of IP addresses
** (including full addresses, if necessary).  It is suitable for
** representing a categorization of the IPv4 address space--for
** example, mapping IP addresses to country codes.
**
** Strictly speaking, it maps IP address to 31-bit unsigned values.
** These values could be used to represent anything desired.
**
** The current implementation provides only lookups to the data
** structure, with the assumption that the structure has been built as
** a disk file by another tool.
*/

#define SKPREFIXMAP_NOT_FOUND (0xFFFFFFFF)

typedef enum skPrefixMap_err_e {
    SKPREFIXMAP_OK = 0,
    SKPREFIXMAP_ERR_ARGS = 1,
    SKPREFIXMAP_ERR_MEMORY = 2,
    SKPREFIXMAP_ERR_IO = 3
} skPrefixMap_err_t;

/* For addresses: key is 32-bit value representing IP address in
 * network byte-order.
 *
 * For protocol/ports: key is a 32-bit value with the (8-bit) protocol
 * number occupying the high 16-bits and the (16-bit) port number (if
 * the protocol is TCP or UDP) occupying the low 16-bits.  The high
 * 8 bits are always zero, since the protocol is an 8-bit value. 
 */

typedef enum skPrefixMap_cont_e {
    SKPREFIXMAP_CONT_ADDR = 0,
    SKPREFIXMAP_CONT_PROTO_PORT = 1
} skPrefixMap_cont_t;

typedef struct skPrefixMap_st skPrefixMap_t;

void skPrefixMapDelete(skPrefixMap_t *map);

skPrefixMap_err_t skPrefixMapRead(skPrefixMap_t **map, FILE *in);

/* Returns the code for a given input, or SKPREFIXMAP_NOT_FOUND */
uint32_t skPrefixMapGet(skPrefixMap_t *map, uint32_t key);

const char *skPrefixMapGetStringForVal(skPrefixMap_t *map, uint32_t val);

const char *skPrefixMapGetString(skPrefixMap_t *map, uint32_t key);

/* Returns the number of words in the dictionary */
uint32_t skPrefixMapGetDictionaryWordCount(skPrefixMap_t *map);

/* Returns the max length of any word in the dictionary */
uint32_t skPrefixMapGetDictionaryMaxWordSize(skPrefixMap_t *map);

/* Returns the code for a given dictionary word (case-insensitive),
   or SKPREFIXMAP_NOT_FOUND */
uint32_t skPrefixMapGetDictionaryVal(skPrefixMap_t *map, const char *word);

/* Returns the content type for a given map */
skPrefixMap_cont_t skPrefixMapGetContentType(skPrefixMap_t *map);

#endif

/*
** Local variables:
** mode:c
** indent-tabs-mode:nil
** c-basic-offset:4
** End:
*/
