/*
** Copyright (C) 2006 by Carnegie Mellon University.
**
** @OPENSOURCE_HEADER_START@
**
** Use of the SILK system and related source code is subject to the terms
** of the following licenses:
**
** GNU Public License (GPL) Rights pursuant to Version 2, June 1991
** Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
**
** NO WARRANTY
**
** ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER
** PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY
** PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN
** "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY
** KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT
** LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE,
** MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE
** OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT,
** SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY
** TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF
** WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES.
** LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF
** CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON
** CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE
** DELIVERABLES UNDER THIS LICENSE.
**
** Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie
** Mellon University, its trustees, officers, employees, and agents from
** all claims or demands made against them (and any related losses,
** expenses, or attorney's fees) arising out of, or relating to Licensee's
** and/or its sub licensees' negligent use or willful misuse of or
** negligent conduct or willful misconduct regarding the Software,
** facilities, or other rights or assistance granted by Carnegie Mellon
** University under this License, including, but not limited to, any
** claims of product liability, personal injury, death, damage to
** property, or violation of any laws or regulations.
**
** Carnegie Mellon University Software Engineering Institute authored
** documents are sponsored by the U.S. Department of Defense under
** Contract F19628-00-C-0003. Carnegie Mellon University retains
** copyrights in all material produced under this contract. The U.S.
** Government retains a non-exclusive, royalty-free license to publish or
** reproduce these documents, or allow others to do so, for U.S.
** Government purposes only pursuant to the copyright license under the
** contract clause at 252.227.7013.
**
** @OPENSOURCE_HEADER_END@
*/
#ifndef _SKSITE_H
#define _SKSITE_H

#include "silk.h"

RCSIDENTVAR(rcsID_SKSITE_H, "$SiLK: sksite.h 5350 2006-10-16 14:43:43Z mthomas $");

#include "rwpack.h"
#include "skvector.h"


/*
**  sksite.h
**
**  An interface to site-specific settings.
**
*/



struct _flowtype_iter;
typedef struct _flowtype_iter flowtype_iter_t;


int sksiteInitialize(int flags);


char *sksiteFlowtypeGetClass(
    char           *out_buffer,
    flowtypeID_t    id,
    size_t          bufsize);
char *sksiteFlowtypeGetClassSafe(
    char           *out_buffer,
    flowtypeID_t    id,
    size_t          bufsize);
/*
 *    Given the flowtype 'id', write the class-name of that flowtype
 *    into 'out_buffer' whose length is 'bufsize'.  Return a pointer
 *    to out_buffer.
 *
 *    If 'id' is an invalid flowtype, or 'out_buffer' is NULL, or
 *    bufsize is too small to hold the class-name, the first function
 *    will return NULL.
 *
 *    The 'Safe' version never returns NULL; on error, it will attempt
 *    to fill 'out_buffer' with the string "?" and return a pointer to
 *    'out_buffer'.  If 'out_buffer' is NULL or too small, the
 *    function returns a pointer to a static string containing "?".
 *
 *    To ensure that 'out_buffer' is large enough to hold all possible
 *    class-names, it's length should be SK_MAX_STRLEN_FLOWTYPE+1.
 */


char *sksiteFlowtypeGetType(
    char           *out_buffer,
    flowtypeID_t    id,
    size_t          bufsize);
char *sksiteFlowtypeGetTypeSafe(
    char           *out_buffer,
    flowtypeID_t    id,
    size_t          bufsize);
/*
 *    Given the flowtype 'id', write the type-name of that flowtype
 *    into 'out_buffer' whose length is 'bufsize'.  Return a pointer
 *    to out_buffer.
 *
 *    If 'id' is an invalid flowtype, or 'out_buffer' is NULL, or
 *    bufsize is too small to hold the type-name, the first function
 *    will return NULL.
 *
 *    The 'Safe' version never returns NULL; on error, it will attempt
 *    to fill 'out_buffer' with the string "?" and return a pointer to
 *    'out_buffer'.  If 'out_buffer' is NULL or too small, the
 *    function returns a pointer to a static string containing "?".
 *
 *    To ensure that 'out_buffer' is large enough to hold all possible
 *    type-names, it's length should be SK_MAX_STRLEN_FLOWTYPE+1.
 */


int sksiteSensorNameIsLegal(const char *name);
/*
 *    Return 0 if the C-string 'name' is of proper form for a sensor
 *    name; i.e., 'name' contains no illegal characters and isn't too
 *    long or too short.
 *
 *    Return a non-zero valid to indicate a problem with 'name': -1 if
 *    'name' does not begin with a alphabetic character; -2 if 'name'
 *    is too short, -3 if 'name' is too long.  A positive return value
 *    indicates that the character at that position in 'name' is
 *    invalid.
 *
 *    To see if 'name' represents an actual sensor, check whether
 *    sksiteLookupSensor(name) returns SK_INVALID_SENSOR.
 */


char *sksiteSensorGetName(
    char           *out_buffer,
    sensorID_t      id,
    size_t          bufsize);
char *sksiteSensorGetNameSafe(
    char           *out_buffer,
    sensorID_t      id,
    size_t          bufsize);
/*
 *    Given the sensor 'id', write the sensor-name of that sensor
 *    into 'out_buffer' whose length is 'bufsize'.  Return a pointer
 *    to 'out_buffer'.
 *
 *    If 'id' is an invalid sensor, or 'out_buffer' is NULL, or
 *    bufsize is too small to hold the sensor-name, the first function
 *    will return NULL.
 *
 *    The 'Safe' version never returns NULL; on error, it will attempt
 *    to fill 'out_buffer' with the string "UNKN?" and return a
 *    pointer to 'out_buffer'.  If 'out_buffer' is NULL or too small,
 *    the function returns a pointer to a static string containing
 *    "UNKN?".
 *
 *    To ensure that out_buffer is large enough to hold all possible
 *    type-names, it's length should be SK_MAX_STRLEN_SENSOR+1.
 */


int sksiteSensorGetClasses(
    uint32_t           *out_class_bitmap,
    sensorID_t          id);
/*
 *    Fill 'out_class_bitmap' with the classes that the sensor 'id'
 *    belongs to---with a high bit indicating that the sensor belongs
 *    to the class---and return the number high bits set in
 *    'out_class_bitmap'.  Return -1 if 'id' is not a valid sensor ID.
 */


int sksiteCreateTemporarySensor(
    const char         *sensor_name,
    const sk_vector_t  *class_names);
/*
 *    Create a temporary sensor whose name is 'sensor_name' and
 *    belongs to the class names (strings) listed in the 'class_names'
 *    vector.  Returns 0 on success.
 *
 *    A positive return value indicates that one of the class names is
 *    invalid; the value is one more than the index of the invalid
 *    class name in the vector 'class_names'.  For example, a return
 *    value of 1 means the first class name---at vector index 0---is
 *    invalid.
 *
 *    The return value -1 indicates that an input was NULL or that
 *    class_names is not a vector of 'char*'s.  The value -2 is
 *    returned when the sensor_name is invalid; i.e.,
 *    sksiteSensorNameIsLegal() returns non-zero; -3 if a sensor of
 *    that name already exists; -4 if the number of class_names is too
 *    small or too large;
 */


int sksiteParseSensorList(
    sk_vector_t        *out_sensors,
    const char         *string);
/*
 *    Treats 'string' as a comma separated list of tokens representing
 *    the NAMES of sensors---does not accept sensor IDs.  For each
 *    token that is a valid sensor name, the function appends an
 *    element to the 'out_sensors' vector, where the element holds the
 *    sensorID_t that the token maps to.
 *
 *    Returns the number of matches found, 0 for no matches, or -1 for
 *    memory error, NULL input, or if the vector's element's size is
 *    not sizeof(sensorID_t).
 *
 *    If out_sensors is NULL, the function still returns the number of
 *    matches found.
 */


char *sksiteFileformatGetName(
    char           *out_buffer,
    fileFormat_t    id,
    size_t          bufsize);
char *sksiteFileformatGetNameSafe(
    char           *out_buffer,
    fileFormat_t    id,
    size_t          bufsize);
/*
 *    Given the file-output-format 'id', write the name of that file
 *    format into 'out_buffer' whose length is 'bufsize'.  Return a
 *    pointer to 'out_buffer'.
 *
 *    If 'id' is an invalid file-output-format, or 'out_buffer' is
 *    NULL, or bufsize is too small to hold the name, the first
 *    function will return NULL.
 *
 *    The 'Safe' version never returns NULL; on error, it will attempt
 *    to fill 'out_buffer' with the string "UNKN?" and return a
 *    pointer to 'out_buffer'.  If 'out_buffer' is NULL or too small,
 *    the function returns a pointer to a static string containing
 *    "UNKN?".
 *
 *    To ensure that out_buffer is large enough to hold all possible
 *    type-names, it's length should be SK_MAX_STRLEN_FILE_FORMAT+1.
 */


int sksiteFileformatIsValid(
    fileFormat_t        id);
/*
 *    Return 1 if 'id' is a valid file output format, or 0 if it is
 *    not.
 */



char *sksiteCompmethodGetName(
    char               *out_buffer,
    sk_compmethod_t     id,
    size_t              bufsize);
/*
 *    Given the compress method 'id', write the name of that method
 *    into 'out_buffer' whose length is 'bufsize'.  The function
 *    returns a pointer to 'out_buffer', or NULL for an invalid
 *    compression method.
 */


int sksiteCompmethodIsAvailable(
    sk_compmethod_t     id);
/*
 *    Return 1 if the compression method 'id' is available, or 0
 *    otherwise.
 *
 *    Since compression is supported by external libraries, a value
 *    may be valid but unavailable when SiLK is built without support
 *    for an external library.
 */


int sksiteCompmethodIsValid(
    sk_compmethod_t     id);
/*
 *    Return 1 if 'id' is a valid compression method, or 0 if it is
 *    not.  See also sksiteCompmethodIsAvailable().
 */


sk_compmethod_t sksiteCompmethodGetBest(void);
/*
 *    Return the generically "best" compression method from all those
 *    that are available.
 */


sk_compmethod_t sksiteCompmethodGetDefault(void);
/*
 *    Return the default compression method.
 */


int sksiteCompmethodOptionsRegister(
    sk_compmethod_t    *compression_method);
/*
 *    Add an option that will allow the user to set the compression
 *    method of binary output files.  After optionsParse() sucessfully
 *    returns, the variable pointed to by 'compression_method' will
 *    contain the compression method to use.
 *
 *    If the user does not exercise the compression-method option,
 *    'compression_method' will be set to the default compression
 *    method.
 */

void sksiteCompmethodOptionsUsage(
    FILE               *fh);
/*
 *    Include the compression method option in the list of options for
 *    --help output.
 */


int sksiteSelectFlowtypesByClassType(
    flowtype_iter_t    *iter,
    const char         *class,
    const char         *type);


flowtypeID_t sksiteLookupFlowtype(
    const char         *class,
    const char         *type);


sensorID_t sksiteLookupSensor(
    const char         *sensor_name);


size_t sksiteGetMaxStrLenClass(void);
/*
 *    Return the string length of the longest class name; does not
 *    include the trailing NUL.
 */


size_t sksiteGetMaxStrLenType(void);
/*
 *    Return the string length of the longest type name; does not
 *    include the trailing NUL.
 */


size_t sksiteGetMaxStrLenSensor(void);
/*
 *    Return the string length of the longest sensor name; does not
 *    include the trailing NUL.
 */


char *sksiteGetRootDir(
    char           *buffer,
    size_t          bufsize);
/*
 *    Fill 'buffer' with the data root directory.  'buffer' is a
 *    character arra whose length is 'bufsize'.  Returns a pointer to
 *    'buffer', or NULL if 'buffer' is too small to hold the root
 *    directory.
 */


int sksiteSetRootDir(
    const char      *rootdir);
/*
 *    Sets the data root directory to 'rootdir', overriding the value
 *    that was set when SiLK was configured.  Returns 0 on success, or
 *    -1 if 'rootdir' is NULL, the empty string, or longer than
 *    PATH_MAX characters.
 */


char *sksiteGeneratePathname(
    char           *buffer,
    size_t          bufsize,
    flowtypeID_t    flowtype,
    sensorID_t      sensor,
    uint32_t        timestamp,
    const char     *suffix,
    char          **reldir_begin,
    char          **filename_begin);
/*
 *    Fill in 'buffer', a C-array of size 'bufsize', with the complete
 *    path of the file having the specified 'flowtype', 'sensor', and
 *    'timestamp'.  The name will be of the form:
 *
 *        /rootdir/<relative_path>/<flowtype>-<sensor>_YYYYMMDD.HH
 *
 *    When 'suffix' is non-NULL and not the empty string, it will be
 *    appended to the name of the generated file.  If 'suffix' does
 *    not begin with a period ("."), one will be added between the
 *    hour and the suffix:
 *
 *        /rootdir/<relative_path>/<flowtype>-<sensor>_YYYYMMDD.HH.<suffix>
 *
 *    When 'reldir_begin' is non-NULL, its value is set to the
 *    beginning of the relative directory; i.e., to the character
 *    following the '/' that ends the root-directory.
 *
 *    When 'filename_begin' is non-NULL, its value is set to the
 *    beginning of the filename; i.e., to the character following the
 *    final '/'.
 */


flowtypeID_t sksiteParseFilename(
    flowtypeID_t   *out_flowtype,
    sensorID_t     *out_sensor,
    uint32_t       *out_timestamp,
    char          **out_suffix,
    const char     *filename);
/*
 *    Extract the flowtype, sensor, and timestamp from 'filename', the
 *    name of a SiLK Flow file, and put the values into
 *    'out_flowtype', 'out_sensor' and 'out_timestamp', respectively.
 *    Any of those values may be NULL, indicating that the caller does
 *    not want the value returned.  The function returns the flowtype,
 *    or SK_INVALID_FLOWTYPE if the name is not in the proper form.
 *
 *    The standard format for the name of a SiLK flow file is:
 *
 *    flowtypeString-SensorName_YYYYMMDD.HH[.*]
 *
 *    with a hyphen separating the flowtype from the sensor, and an
 *    underscore separating the sensor from the time.  The filename
 *    may include a suffix made up of a period and additional text.
 *    When 'filename' has a suffix and 'out_suffix' is non-NULL,
 *    'out_suffix' will be set to the location of the '.' in
 *    'filename' that begins the suffix.
 *
 *    The function will take the basename of 'filename', so 'filename'
 *    may include directory compontents.
 *
 */


char *sksiteParseGeneratePath(
    char           *buffer,
    size_t          bufsize,
    const char     *filename,
    const char     *suffix,
    char          **reldir_begin,
    char          **filename_begin);
/*
 *    Extract the flowtype, sensor, and timestamp from 'filename', the
 *    name of a SiLK Flow file, and use those values to fill in
 *    'buffer', a C-array of size 'bufsize', with the complete path of
 *    the file having that flowtype, sensor, and timestamp.
 *
 *    Returns a pointer to 'buffer' on success, or NULL if an error
 *    occurs or if the filename is not of the expected form.
 *
 *    When 'suffix' is NULL and 'filename' has a suffix, the path name
 *    put into 'buffer' will have the same suffix appended.  To have
 *    the suffix removed from the new path, pass in the empty string
 *    as the value of 'suffix'.
 *
 *    The parameters 'buffer' and 'filename' may point to the same
 *    location.
 *
 *    This function works as a convenience wrapper around
 *    sksiteParseFilename() and sksiteGeneratePathname().  See the
 *    documentation of those functions for additional information.
 */

#endif /* _SKSITE_H */
