/*
** Copyright (C) 2006 by Carnegie Mellon University.
**
** @OPENSOURCE_HEADER_START@
** 
** Use of the SILK system and related source code is subject to the terms 
** of the following licenses:
** 
** GNU Public License (GPL) Rights pursuant to Version 2, June 1991
** Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
** 
** NO WARRANTY
** 
** ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER 
** PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY 
** PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN 
** "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY 
** KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT 
** LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE, 
** MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE 
** OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT, 
** SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY 
** TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF 
** WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES. 
** LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF 
** CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON 
** CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE 
** DELIVERABLES UNDER THIS LICENSE.
** 
** Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie 
** Mellon University, its trustees, officers, employees, and agents from 
** all claims or demands made against them (and any related losses, 
** expenses, or attorney's fees) arising out of, or relating to Licensee's 
** and/or its sub licensees' negligent use or willful misuse of or 
** negligent conduct or willful misconduct regarding the Software, 
** facilities, or other rights or assistance granted by Carnegie Mellon 
** University under this License, including, but not limited to, any 
** claims of product liability, personal injury, death, damage to 
** property, or violation of any laws or regulations.
** 
** Carnegie Mellon University Software Engineering Institute authored 
** documents are sponsored by the U.S. Department of Defense under 
** Contract F19628-00-C-0003. Carnegie Mellon University retains 
** copyrights in all material produced under this contract. The U.S. 
** Government retains a non-exclusive, royalty-free license to publish or 
** reproduce these documents, or allow others to do so, for U.S. 
** Government purposes only pursuant to the copyright license under the 
** contract clause at 252.227.7013.
** 
** @OPENSOURCE_HEADER_END@
*/
#ifndef _YAFSOURCE_H
#define _YAFSOURCE_H

#include "silk.h"

RCSIDENTVAR(rcsID_YAFSOURCE_H, "$SiLK: yafsource.h 5108 2006-09-29 15:48:10Z mwd $");

/*
**  yafsource.h
**
*/


#include "flowcap-stats.h"
#include "flowcap.h"
#include "rwpack.h"

/* A YAF source is a record source based on YAF records.  Once
   created, records can be requested of it via a pull mechanism. */

/* The type of YAF sources. */
typedef struct yaf_source_t *yafSource_t;

/* Creates a YAF source.  

   port is the tcp port to listen upon for YAF flows.
   listen_on_address is the address to bind to.  max_flows is the
   number of YAF flows the created source can buffer in memory.  logfn
   is a function for logging errors and such.  It can be NULL.

   Returns a YAF source on success, or NULL on failure. */
yafSource_t yafSourceCreate(
    int port,
    in_addr_t listen_on_address,
    uint32_t max_flows,
    sk_msg_fn_t logfn);

/* Destroys a YAF source. */
void yafSourceDestroy(
    yafSource_t source);


/* Requests a generic record from a YAF source.

   This function will block if there are no YAF flows available from
   which to create a generic record.

   Returns 0 on success, -1 on failure. */
int yafSourceGetGeneric(
    yafSource_t source, 
    rwRec *rec);

/* Requests a flowcap record from a YAF source.  

   fc_version determines what version of flowcap the function will
   produce.  This function will block if there are no YAF flows
   available from which to create a flowcap record.

   Returns 0 on success, -1 on failure. */
int yafSourceGetFlowcap(
    yafSource_t source, 
    flowcapRec_t *rec, 
    int fc_version);

#endif /* _YAFSOURCE_H */
