/*
** Copyright (C) 2006 by Carnegie Mellon University.
**
** @OPENSOURCE_HEADER_START@
**
** Use of the SILK system and related source code is subject to the terms
** of the following licenses:
**
** GNU Public License (GPL) Rights pursuant to Version 2, June 1991
** Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
**
** NO WARRANTY
**
** ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER
** PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY
** PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN
** "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY
** KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT
** LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE,
** MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE
** OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT,
** SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY
** TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF
** WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES.
** LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF
** CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON
** CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE
** DELIVERABLES UNDER THIS LICENSE.
**
** Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie
** Mellon University, its trustees, officers, employees, and agents from
** all claims or demands made against them (and any related losses,
** expenses, or attorney's fees) arising out of, or relating to Licensee's
** and/or its sub licensees' negligent use or willful misuse of or
** negligent conduct or willful misconduct regarding the Software,
** facilities, or other rights or assistance granted by Carnegie Mellon
** University under this License, including, but not limited to, any
** claims of product liability, personal injury, death, damage to
** property, or violation of any laws or regulations.
**
** Carnegie Mellon University Software Engineering Institute authored
** documents are sponsored by the U.S. Department of Defense under
** Contract F19628-00-C-0003. Carnegie Mellon University retains
** copyrights in all material produced under this contract. The U.S.
** Government retains a non-exclusive, royalty-free license to publish or
** reproduce these documents, or allow others to do so, for U.S.
** Government purposes only pursuant to the copyright license under the
** contract clause at 252.227.7013.
**
** @OPENSOURCE_HEADER_END@
*/
#ifndef _CONVERT_YAF_H
#define _CONVERT_YAF_H

#include "silk.h"

RCSIDENTVAR(rcsID_CONVERT_YAF_H, "$SiLK: convert-yaf.h 5108 2006-09-29 15:48:10Z mwd $");

/*
**  convert-yaf.h
**
**  Converts from YAF records
**
*/

#include <yaf/yafcore.h>
#include "flowcap.h"
#include "rwpack.h"

typedef enum {
    YAF_FORWARD = 0,
    YAF_REVERSE = 1 } yaf_dir_t;

/* Fills out a flowcap record from a YAF flow. 

   yafPtr:      	A pointer to the yaf flow object
   direction:           Either YAF_FORWARD or YAF_REVERSE.  Should
                        represent which half of a YAF biflow to
                        convert.
   flowcap_rec:         A pointer to the flowcap record to fill out
   flowcap_version:     The flowcap version requested

   Returns -1 if YAF_REVERSE and there is no reverse record.  Returns
   0 otherwise.
 */
int yafToFlowcap(
    const YAFlow   *yafPtr,
    yaf_dir_t       direction,
    flowcapRec_t   *flowcap_rec,
    int             flowcap_version);

/* Fills out an rwGeneric record from a YAF flow. 

   yafPtr:      	A pointer to the yaf flow object
   direction:           Either YAF_FORWARD or YAF_REVERSE.  Should
                        represent which half of a YAF biflow to
                        convert.
   rwrec:               A pointer to the rwGeneric record to fill out

   Returns -1 if YAF_REVERSE and there is no reverse record.  Returns
   0 otherwise.
 */
int yafToGeneric(
    const YAFlow   *yafPtr,
    yaf_dir_t       direction,
    rwRec          *rwrec);


#endif /* _CONVERT_YAF_H */


/*
** Local variables:
** mode:c
** indent-tabs-mode:nil
** c-basic-offset:4
** End:
*/
