/*
** Copyright (C) 2004-2006 by Carnegie Mellon University.
**
** @OPENSOURCE_HEADER_START@
** 
** Use of the SILK system and related source code is subject to the terms 
** of the following licenses:
** 
** GNU Public License (GPL) Rights pursuant to Version 2, June 1991
** Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
** 
** NO WARRANTY
** 
** ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER 
** PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY 
** PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN 
** "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY 
** KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT 
** LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE, 
** MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE 
** OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT, 
** SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY 
** TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF 
** WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES. 
** LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF 
** CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON 
** CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE 
** DELIVERABLES UNDER THIS LICENSE.
** 
** Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie 
** Mellon University, its trustees, officers, employees, and agents from 
** all claims or demands made against them (and any related losses, 
** expenses, or attorney's fees) arising out of, or relating to Licensee's 
** and/or its sub licensees' negligent use or willful misuse of or 
** negligent conduct or willful misconduct regarding the Software, 
** facilities, or other rights or assistance granted by Carnegie Mellon 
** University under this License, including, but not limited to, any 
** claims of product liability, personal injury, death, damage to 
** property, or violation of any laws or regulations.
** 
** Carnegie Mellon University Software Engineering Institute authored 
** documents are sponsored by the U.S. Department of Defense under 
** Contract F19628-00-C-0003. Carnegie Mellon University retains 
** copyrights in all material produced under this contract. The U.S. 
** Government retains a non-exclusive, royalty-free license to publish or 
** reproduce these documents, or allow others to do so, for U.S. 
** Government purposes only pursuant to the copyright license under the 
** contract clause at 252.227.7013.
** 
** @OPENSOURCE_HEADER_END@
*/
#ifndef _SKPREFIXMAP_PRIV_H
#define _SKPREFIXMAP_PRIV_H

#include "skprefixmap.h"

RCSIDENTVAR(rcsID_SKPREFIXMAP_PRIV_H, "$SiLK: skprefixmap_priv.h 4248 2006-07-11 20:40:53Z prevost1 $");

/*
** prefixmap_priv.h
**
** John McClary Prevost
** December 2nd, 2004
**
** Private definitions for prefixmap data structure implementation.
*/

/* skPrefixMap header is silkHeader followed by a different header
 * depending on version. */

typedef struct skPrefixMap_header_st {
    genericHeader silkHeader;
    uint32_t recordCount;	/* Number of l/r records */
} skPrefixMap_header_t;
/* v1: followed by recordCount x skPrefixMap_record_t */
/* v2: followed by recordCount x skPrefixMap_record_t, then followed by
       uint32_t dictionarySize, and dictionarySize x char */

typedef struct skPrefixMap_record_st {
    uint32_t left;
    uint32_t right;
} skPrefixMap_record_t;

struct skPrefixMap_st {
    skPrefixMap_cont_t contentType;
    size_t bufferSize;
    skPrefixMap_record_t *buffer;
    uint32_t dictionarySize;	/* 0 for version 1 */
    char *dictionary;		/* NULL for version 1 */
    uint32_t dictionaryWordCount; /* Number of words in dict, 0 for v1 */
    char **dictionaryWords;	/* NULL for version 1 */
    uint32_t dictionaryWordMaxSize; /* Max len of word in dict, 0 for v1 */
};

#define SKPREFIXMAP_IS_NODE(x) (!(x & 0x80000000))
#define SKPREFIXMAP_IS_LEAF(x) (x & 0x80000000)
#define SKPREFIXMAP_LEAF_VALUE(x) (x & 0x7FFFFFFF)

#endif /* _SKPREFIXMAP_PRIV_H */
