/*
** Copyright (C) 2006 by Carnegie Mellon University.
**
** @OPENSOURCE_HEADER_START@
**
** Use of the SILK system and related source code is subject to the terms
** of the following licenses:
**
** GNU Public License (GPL) Rights pursuant to Version 2, June 1991
** Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
**
** NO WARRANTY
**
** ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER
** PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY
** PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN
** "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY
** KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT
** LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE,
** MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE
** OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT,
** SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY
** TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF
** WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES.
** LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF
** CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON
** CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE
** DELIVERABLES UNDER THIS LICENSE.
**
** Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie
** Mellon University, its trustees, officers, employees, and agents from
** all claims or demands made against them (and any related losses,
** expenses, or attorney's fees) arising out of, or relating to Licensee's
** and/or its sub licensees' negligent use or willful misuse of or
** negligent conduct or willful misconduct regarding the Software,
** facilities, or other rights or assistance granted by Carnegie Mellon
** University under this License, including, but not limited to, any
** claims of product liability, personal injury, death, damage to
** property, or violation of any laws or regulations.
**
** Carnegie Mellon University Software Engineering Institute authored
** documents are sponsored by the U.S. Department of Defense under
** Contract F19628-00-C-0003. Carnegie Mellon University retains
** copyrights in all material produced under this contract. The U.S.
** Government retains a non-exclusive, royalty-free license to publish or
** reproduce these documents, or allow others to do so, for U.S.
** Government purposes only pursuant to the copyright license under the
** contract clause at 252.227.7013.
**
** @OPENSOURCE_HEADER_END@
*/

/*
**  Add your description here
**
*/


#define DECLARE_CLASSINFO_VARIABLES  1
#define DEFINE_CLASSINFO_VARIABLES   1
#define DECLARE_FILEFORMAT_VARIABLES 1
#define DEFINE_FILEFORMAT_VARIABLES  1
#define DECLARE_SITE_VARIABLES       1
#define DEFINE_SITE_VARIABLES        1
#define DECLARE_SENSORID_VARIABLES   1
#define DEFINE_SENSORID_VARIABLES    1
#include "silk.h"

RCSIDENT("$SiLK: sksite.c 5704 2006-11-15 14:47:15Z mthomas $");

#include "sksite.h"
#include "utils.h"
#include "skvector.h"


static char data_rootdir[PATH_MAX];


/* If TEST_SITE is defined; some macros are redefined as functions */
/* #define TEST_SITE 1 */


#define SK_INVALID_CLASS   UINT8_MAX
#define SK_INVALID_TYPE    UINT8_MAX


typedef struct _pool_struct {
    /* the data */
    void               *p_data;
    /* the next element to return */
    uint8_t            *p_next_elem;
    /* the growth factor for the array */
    double              p_factor;
    /* the number of elements handed out */
    size_t              p_count;
    /* the allocated size of the array */
    size_t              p_alloc_size;
    /* the length of each element */
    size_t              p_elem_size;
} pool_struct_t;

#define POOL_DATA(p)                ((p).p_data)
#define POOL_COUNT(p)               ((p).p_count)
#define POOL_ALLOC_SIZE(p)          ((p).p_alloc_size)
#define POOL_ELEM_SIZE(p)           ((p).p_elem_size)


typedef struct _symbol_struct {
    /* name of symbol */
    const char         *sym_name;
    /* the symbol's length */
    size_t              sym_strlen;
    /* the symbol's id--must be its position in the array */
    uint8_t             sym_id;
} symbol_struct_t;


typedef struct _sym_list {
    /* the data */
    pool_struct_t       syl_pool;
    /* length of longest symbol */
    size_t              syl_strlen_max;
} sym_list_t;

#define SYMLIST_MAX_STRLEN(sl)      ((sl).syl_strlen_max)
#define SYMLIST_COUNT(sl)           (POOL_COUNT((sl).syl_pool))
#define SYMLIST_ELEMENT(sl, i)                          \
    (((symbol_struct_t*)POOL_DATA(sl.syl_pool))[(i)])


typedef struct _flowtype_struct {
    /* unique name for this flowtype: 'in', 'inweb', etc */
    const char         *ft_name;
    /* relative directory for files of this flowtype */
    const char         *ft_rel_dir;
    /* the flowtype's id--must be its position in the array */
    flowtypeID_t        ft_id;
    /* file format to use for files of this flowtype */
    fileFormat_t        ft_format;
    /* file version to use for files of this flowtype */
    fileVersion_t       ft_version;
    /* the class symbol */
    uint8_t             ft_class;
    /* the type symbol */
    uint8_t             ft_type;
    /* is this the default class? */
    unsigned            ft_default_class : 1;
    /* is this a default flowtype for this class? */
    unsigned            ft_default_type_for_class :1;
} flowtype_struct_t;


typedef struct _sensor_struct {
    /* unique name for this sensor */
    const char         *s_name;
    /* length of the name */
    size_t              s_strlen;
    /* number of classes it belongs to */
    size_t              s_class_count;
    /* bitmap of classes it belongs to */
    uint32_t            s_class_map;
    /* the sensor's id--must be its position in the array */
    sensorID_t          s_id;
} sensor_struct_t;


typedef struct _sensor_list {
    /* the data */
    pool_struct_t       sl_pool;
    /* length of longest symbol */
    size_t              sl_strlen_max;
} sensor_list_t;


typedef struct _fileformat_struct {
    /* unique name of this output format */
    const char         *ff_name;
    /* the output format's id--must be its position in the array */
    fileFormat_t        ff_id;
} fileformat_struct_t;


typedef struct _compmethod_struct {
    /* unique name of this compression method */
    const char         *cm_name;
    /* the compression method's id--must be its position in the array */
    sk_compmethod_t     cm_id;
} compmethod_struct_t;



/*
 *  *****  POOL  *****
 */


static int _sitePoolCreate(
    pool_struct_t      *pool,
    size_t              alloc_sz,
    size_t              elem_sz,
    double              grow_factor)
{
    assert(pool);
    assert(alloc_sz);
    assert(elem_sz);

    pool->p_data = pool->p_next_elem = calloc(alloc_sz, elem_sz);
    pool->p_count = 0;
    pool->p_alloc_size = alloc_sz;
    pool->p_elem_size = elem_sz;
    if (grow_factor < 1.0) {
        grow_factor = 2.0;
    }
    pool->p_factor = grow_factor;
    if (pool->p_data == NULL) {
        skAppPrintErr("Out of memory!");
        abort();
    }
    return 0;
}


static void *_sitePoolAllocElement(
    pool_struct_t      *pool)
{
    void *data;

    assert(pool);
    assert(pool->p_count <= pool->p_alloc_size);

    if (pool->p_count == pool->p_alloc_size) {
        /* need to grow the pool */
        void *old_data = pool->p_data;
        size_t old_size = pool->p_alloc_size;

        pool->p_alloc_size = pool->p_factor * (double)pool->p_alloc_size;
        if (pool->p_alloc_size <= old_size) {
            skAppPrintErr("Bad growth factor or initial array too small");
            abort();
        }
        pool->p_data = realloc(old_data, (pool->p_alloc_size
                                          * pool->p_elem_size));
        if ( !pool->p_data) {
            pool->p_alloc_size = old_size;
            pool->p_data = old_data;
            skAppPrintErr("Out of memory!");
            abort();
        }

        /* reset the next element since the data may have moved */
        pool->p_next_elem = (((uint8_t*)pool->p_data)
                             + pool->p_elem_size * old_size);

        /* clear the newly allocated memory */
        memset(pool->p_next_elem, 0,
               pool->p_elem_size * (pool->p_alloc_size - old_size));
    }

    /* the value to return */
    data = (void*)(pool->p_next_elem);

    /* move p_next_elem to the next element to be returned */
    pool->p_next_elem += pool->p_elem_size;
    ++pool->p_count;

    return data;
}


/*
 *  *****  CLASSES  *****
 */


static sym_list_t class_list;


#ifdef TEST_SITE

static size_t _siteGetMaxStrlenClasssym(void)
{
    return SYMLIST_MAX_STRLEN(class_list);
}

static uint8_t _siteGetClasssymCount(void)
{
    return SYMLIST_COUNT(class_list);
}

static const char *_siteClasssymGetNameFast(uint8_t symid)
{
    return SYMLIST_ELEMENT(class_list, symid).sym_name;
}

static size_t _siteClasssymGetStrlenFast(uint8_t symid)
{
    return SYMLIST_ELEMENT(class_list, symid).sym_strlen;
}

#else /* TEST_SITE */

#define _siteGetMaxStrlenClasssym()             \
    (SYMLIST_MAX_STRLEN(class_list))

#define _siteGetClasssymCount()                 \
    (SYMLIST_COUNT(class_list))

#define _siteClasssymGetNameFast(symid)         \
    (SYMLIST_ELEMENT(class_list, symid).sym_name)

#define _siteClasssymGetStrlenFast(symid)       \
    (SYMLIST_ELEMENT(class_list, symid).sym_strlen)

#endif /* TEST_SITE */


/*
 *  ok = _siteInitializeClass(init_size);
 *
 *    Initialize the data structures used by classes; 'init_size' is
 *    the estimated number of classes.
 */
static int _siteInitializeClass(size_t sz)
{
    memset(&class_list, 0, sizeof(sym_list_t));
    _sitePoolCreate(&class_list.syl_pool, sz, sizeof(symbol_struct_t), 2);
    return 0;
}


/*
 *  id = _siteFindClassSym(string);
 *
 *    Return the ID of the class whose name is 'string'; return
 *    SK_INVALID_CLASS if no such class exists.
 */
static uint8_t _siteFindClassSym(
    const char         *string)
{
    uint8_t j;

    assert(string);

    for (j = 0; j < _siteGetClasssymCount(); ++j) {
        if (0 == strcmp(string, _siteClasssymGetNameFast(j))) {
            return j;
        }
    }

    /* not found */
    return SK_INVALID_CLASS;
}


/*
 *  id = sksiteAddClassFast(name);
 *
 *    Add a new class whose name is 'name' and return the id of the
 *    class.  Do not check to see if a class with 'name' already
 *    exists.
 */
static uint8_t sksiteAddClassFast(const char *class_name)
{
    symbol_struct_t *ssi;
    uint8_t class_id = SYMLIST_COUNT(class_list);
    size_t class_len = strlen(class_name);

    /* check length of class name */
    if (class_len > SK_MAX_STRLEN_FLOWTYPE) {
        skAppPrintErr(("Class name '%s' is longer than allowed (%u).\n"
                       "\tFix your site header and recompile.  Abort!"),
                      class_name, SK_MAX_STRLEN_FLOWTYPE);
        assert(class_len <= SK_MAX_STRLEN_FLOWTYPE);
        abort();
    }

    /* get a new element */
    ssi = _sitePoolAllocElement(&class_list.syl_pool);

    /* fill in the element */
    ssi->sym_name = class_name;
    ssi->sym_strlen = class_len;
    ssi->sym_id = class_id;

    /* is this the longest name? */
    if (ssi->sym_strlen > SYMLIST_MAX_STRLEN(class_list)) {
        class_list.syl_strlen_max = ssi->sym_strlen;
    }

    return class_id;
}


/*
 *  id = sksiteLookupAddClass(name);
 *
 *    Return the id of the class whose name is 'name', creating the
 *    class if it does not exist.
 */
static uint8_t sksiteLookupAddClass(const char *class_name)
{
    uint8_t i;

    /* if it already exists, return it */
    for (i = 0; i < _siteGetClasssymCount(); ++i) {
        if (0 == strcmp(_siteClasssymGetNameFast(i), class_name)) {
            return i;
        }
    }

    return sksiteAddClassFast(class_name);
}


/*
 *  length = sksiteGetMaxStrLenClass();
 *
 *    Return the length (strlen()) of the longest class name.
 */
size_t sksiteGetMaxStrLenClass(void)
{
    return _siteGetMaxStrlenClasssym();
}


/*
 *  match_count = _siteParseClassList(out_classs, class_list);
 *
 *    Treats 'class_list' as a comma separated list of tokens
 *    representing the 'class' string for flowtypes.  Each token is
 *    compared against the list of known class-symbols stored in the
 *    global 'class_sym'.  When a match is found, that token's id in
 *    the out_class array is set to 1.
 *
 *    Returns the number of matches found, 0 for no matches, or -1 for
 *    memory error.
 */
static int _siteParseClassList(uint8_t *class_seen, const char *string)
{
    int seen_count = 0;
    char *str_copy = NULL;
    char *cp;
    char *ep;
    uint8_t id;
    int rv = -1;

    /* clear the array of values we will set */
    memset(class_seen, 0, (_siteGetClasssymCount() * sizeof(uint8_t)));

    /* make a copy of the input we can modify */
    str_copy = strdup(string);
    if (!str_copy) {
        goto END;
    }

    /* go over the list, converting ',' to '\0', then looking for a
     * class with the given name */
    cp = str_copy;
    while (*cp) {
        ep = strchr(cp, ',');
        if (ep == cp) {
            /* double comma, ignore */
            ++cp;
            continue;
        }
        if (ep != NULL) {
            *ep = '\0';
            ++ep;
        } else {
            /* force ep onto final '\0' */
            ep = cp + strlen(cp);
        }

        id = _siteFindClassSym(cp);
        if (id == SK_INVALID_CLASS) {
            goto END;
        }
        if (0 == class_seen[id]) {
            /* not previously seen */
            class_seen[id] = 1;
            ++seen_count;
        }

        cp = ep;
    }

    /* success */
    rv = seen_count;

  END:
    if (str_copy) {
        free(str_copy);
    }

    return rv;
}


#ifdef TEST_SITE
#  define class_list
#endif



/*
 *  *****  TYPES  *****
 */


static sym_list_t type_list;


#ifdef TEST_SITE

static size_t _siteGetMaxStrlenTypesym(void)
{
    return SYMLIST_MAX_STRLEN(type_list);
}

static uint8_t _siteGetTypesymCount(void)
{
    return SYMLIST_COUNT(type_list);
}

static const char *_siteTypesymGetNameFast(uint8_t symid)
{
    return SYMLIST_ELEMENT(type_list, symid).sym_name;
}

static size_t _siteTypesymGetStrlenFast(uint8_t symid)
{
    return SYMLIST_ELEMENT(type_list, symid).sym_strlen;
}

#else /* TEST_SITE */

#define _siteGetMaxStrlenTypesym()             \
    (SYMLIST_MAX_STRLEN(type_list))

#define _siteGetTypesymCount()                 \
    (SYMLIST_COUNT(type_list))

#define _siteTypesymGetNameFast(symid)         \
    (SYMLIST_ELEMENT(type_list, symid).sym_name)

#define _siteTypesymGetStrlenFast(symid)       \
    (SYMLIST_ELEMENT(type_list, symid).sym_strlen)

#endif /* TEST_SITE */


/*
 *  ok = _siteInitializeType(init_size);
 *
 *    Initialize the data structures used by types; 'init_size' is
 *    the estimated number of types.
 */
static int _siteInitializeType(size_t sz)
{
    memset(&type_list, 0, sizeof(sym_list_t));
    _sitePoolCreate(&type_list.syl_pool, sz, sizeof(symbol_struct_t), 2);
    return 0;
}


/*
 *  id = _siteFindTypeSym(string);
 *
 *    Return the ID of the type whose name is 'string'; return
 *    SK_INVALID_TYPE if no such type exists.
 */
static uint8_t _siteFindTypeSym(
    const char         *string)
{
    uint8_t j;

    assert(string);

    for (j = 0; j < _siteGetTypesymCount(); ++j) {
        if (0 == strcmp(string, _siteTypesymGetNameFast(j))) {
            return j;
        }
    }

    /* not found */
    return SK_INVALID_TYPE;
}


/*
 *  id = sksiteAddTypeFast(name);
 *
 *    Add a new type whose name is 'name' and return the id of the
 *    type.  Do not check to see if a type with 'name' already
 *    exists.
 */
static uint8_t sksiteAddTypeFast(const char *type_name)
{
    symbol_struct_t *ssi;
    uint8_t type_id = SYMLIST_COUNT(type_list);
    size_t type_len = strlen(type_name);

    /* check length of type name */
    if (type_len > SK_MAX_STRLEN_FLOWTYPE) {
        skAppPrintErr(("Type name '%s' is longer than allowed (%u).\n"
                       "\tFix your site header and recompile.  Abort!"),
                      type_name, SK_MAX_STRLEN_FLOWTYPE);
        assert(type_len <= SK_MAX_STRLEN_FLOWTYPE);
        abort();
    }

    /* get a new element */
    ssi = _sitePoolAllocElement(&type_list.syl_pool);

    /* fill in the element */
    ssi->sym_name = type_name;
    ssi->sym_strlen = type_len;
    ssi->sym_id = type_id;

    /* is this the longest name? */
    if (ssi->sym_strlen > SYMLIST_MAX_STRLEN(type_list)) {
        type_list.syl_strlen_max = ssi->sym_strlen;
    }

    return type_id;
}


/*
 *  id = sksiteLookupAddType(name);
 *
 *    Return the id of the type whose name is 'name', creating the
 *    type if it does not exist.
 */
static uint8_t sksiteLookupAddType(const char *type_name)
{
    uint8_t i;

    /* if it already exists, return it */
    for (i = 0; i < _siteGetTypesymCount(); ++i) {
        if (0 == strcmp(_siteTypesymGetNameFast(i), type_name)) {
            return i;
        }
    }

    return sksiteAddTypeFast(type_name);
}


/*
 *  length = sksiteGetMaxStrLenType();
 *
 *    Return the length (strlen()) of the longest type name.
 */
size_t sksiteGetMaxStrLenType(void)
{
    return _siteGetMaxStrlenTypesym();
}


/*
 *  match_count = _siteParseTypeList(out_types, type_list);
 *
 *    Treats 'type_list' as a comma separated list of tokens
 *    representing the 'type' string for flowtypes.  Each token is
 *    compared against the list of known type-symbols stored in the
 *    global 'type_sym'.  When a match is found, that token's id in
 *    the out_type array is set to 1.
 *
 *    Returns the number of matches found, 0 for no matches, or -1 for
 *    memory error.
 */
static int _siteParseTypeList(uint8_t *type_seen, const char *string)
{
    int seen_count = 0;
    char *str_copy = NULL;
    char *cp;
    char *ep;
    uint8_t id;
    int rv = -1;

    /* clear the array of values we will set */
    memset(type_seen, 0, (_siteGetTypesymCount() * sizeof(uint8_t)));

    /* make a copy of the input we can modify */
    str_copy = strdup(string);
    if (!str_copy) {
        goto END;
    }

    /* go over the list, converting ',' to '\0', then looking for a
     * type with the given name */
    cp = str_copy;
    while (*cp) {
        ep = strchr(cp, ',');
        if (ep == cp) {
            /* double comma, ignore */
            ++cp;
            continue;
        }
        if (ep != NULL) {
            *ep = '\0';
            ++ep;
        } else {
            /* force ep onto final '\0' */
            ep = cp + strlen(cp);
        }

        id = _siteFindTypeSym(cp);
        if (id == SK_INVALID_TYPE) {
            goto END;
        }
        if (0 == type_seen[id]) {
            /* not previously seen */
            type_seen[id] = 1;
            ++seen_count;
        }

        cp = ep;
    }

    /* success */
    rv = seen_count;

  END:
    if (str_copy) {
        free(str_copy);
    }

    return rv;
}


#ifdef TEST_SITE
#  define type_list
#endif



/*
 *  *****  SENSORS  *****
 */


static sensor_list_t sensor_list;

#ifdef TEST_SITE

static size_t _siteGetMaxStrlenSensor(void)
{
    return sensor_list.sl_strlen_max;
}

static size_t _siteGetSensorCount(void)
{
    return sensor_list.sl_pool.p_count;
}

static sensor_struct_t *_siteGetSensorInfoFast(sensorID_t sid)
{
    return (&(((sensor_struct_t*)POOL_DATA(sensor_list.sl_pool))[(sid)]));
}

static sensor_struct_t *_siteGetSensorInfo(sensorID_t sid)
{
    return (((sid) >= _siteGetSensorCount())
            ? NULL
            : _siteGetSensorInfoFast(sid));
}

static const char *_siteSensorInfoGetName(const sensor_struct_t *si)
{
    return ((si)->s_name);
}

static size_t _siteSensorInfoGetStrlen(const sensor_struct_t *si)
{
    return ((si)->s_strlen);
}

#else /* TEST_SITE */

#define _siteGetMaxStrlenSensor()               \
    (sensor_list.sl_strlen_max)

#define _siteGetSensorCount()                   \
    (sensor_list.sl_pool.p_count)

#define _siteGetSensorInfoFast(sid)             \
    (&(((sensor_struct_t*)POOL_DATA(sensor_list.sl_pool))[(sid)]))

#define _siteGetSensorInfo(sid)                 \
    (((sid) >= _siteGetSensorCount())           \
     ? NULL                                     \
     : _siteGetSensorInfoFast(sid))

#define _siteSensorInfoGetName(si)              \
    ((si)->s_name)

#define _siteSensorInfoGetStrlen(si)            \
    ((si)->s_strlen)

#endif /* TEST_SITE */


static int _siteInitializeSensor(size_t sz)
{
    memset(&sensor_list, 0, sizeof(sensor_list_t));
    _sitePoolCreate(&sensor_list.sl_pool, sz, sizeof(sensor_struct_t), 2);
    return 0;
}


static sensorID_t sksiteAddSensor(
    const char *sensor_name,
    uint32_t    sensor_id)
{
    sensor_struct_t *si;

    /* check range of sensor_id */
    if (sensor_id >= SK_INVALID_SENSOR) {
        skAppPrintErr(("Sensor id %u is not legal.\n"
                       "\tFix your site header and recompile.  Abort!"),
                      sensor_id);
        assert(sensor_id < SK_INVALID_SENSOR);
        abort();
    }

    /* check for duplicate ID */
    si = _siteGetSensorInfo(sensor_id);
    if (si != NULL) {
        skAppPrintErr(("Sensor with id=%u already exists.\n"
                       "\tFix your site header and recompile.  Abort!"),
                      sensor_id);
        assert(sensor_id < _siteGetSensorCount());
        abort();
    }

    /* check legality of sensor name */
    if (sksiteSensorNameIsLegal(sensor_name)) {
        skAppPrintErr(("Sensor name '%s' is not legal\n"
                       "\tFix your site header and recompile.  Abort!"),
                      sensor_name);
        abort();
    }

    /* get a new element */
    si = _sitePoolAllocElement(&sensor_list.sl_pool);

    /* fill in the element */
    si->s_id = (sensorID_t)sensor_id;
    si->s_name = sensor_name;
    si->s_strlen = strlen(si->s_name);

    /* is this the longest sensor name? */
    if (si->s_strlen > _siteGetMaxStrlenSensor()) {
        sensor_list.sl_strlen_max = si->s_strlen;
    }

    return si->s_id;
}


#ifdef TEST_SITE
#  define sensor_list
#endif


static sensorID_t _siteFindSensorName(
    const char         *name)
{
    sensorID_t i;

    for (i = 0; i < _siteGetSensorCount(); ++i) {
        if (strcmp(_siteSensorInfoGetName(_siteGetSensorInfoFast(i)), name)
            == 0)
        {
            return i;
        }
    }
    return SK_INVALID_SENSOR;
}


sensorID_t sksiteLookupSensor(
    const char         *string)
{
    if (string == NULL) {
        return SK_INVALID_SENSOR;
    }
    return _siteFindSensorName(string);
}


/* Put the name of the sensor into 'out_buffer' and return pointer to
 * that buffer.  Return NULL on error. */
char *sksiteSensorGetName(
    char           *out_buffer,
    sensorID_t      id,
    size_t          bufsize)
{
    sensor_struct_t *si;

    if (out_buffer == NULL) {
        return NULL;
    }
    si = _siteGetSensorInfo(id);
    if (si == NULL) {
        return NULL;
    }
    if (bufsize <= _siteSensorInfoGetStrlen(si)) {
        return NULL;
    }

    strncpy(out_buffer, _siteSensorInfoGetName(si), bufsize);
    out_buffer[bufsize-1] = '\0';
    return out_buffer;
}


char *sksiteSensorGetNameSafe(
    char           *out_buffer,
    sensorID_t      id,
    size_t          bufsize)
{
    static const char unkn[] = "UNKN?";
    char *p = sksiteSensorGetName(out_buffer, id, bufsize);
    if (p) {
        return p;
    }

    if (out_buffer && bufsize > 1+sizeof(unkn)) {
        strncpy(out_buffer, unkn, 1+sizeof(unkn));
        return out_buffer;
    }
    return (char*)unkn;
}


size_t sksiteGetMaxStrLenSensor(void)
{
    return _siteGetMaxStrlenSensor();
}


int sksiteSensorNameIsLegal(const char *name)
{
    size_t len;
    const char *cp;

    /* check length; should be between 1 and SK_MAX_STRLEN_SENSOR */
    len = strlen(name);
    if (len < 1) {
        /* skAppPrintErr("sensor name must contain at least 1 character"); */
        return -2;
    }
    if (len > SK_MAX_STRLEN_SENSOR) {
        /*skAppPrintErr("sensor name can contain no more than %d characters",
         *              SK_MAX_STRLEN_SENSOR);
         */
        return -3;
    }

    /* check for bad characters */
    for (cp = name; *cp; ++cp) {
        if (isspace((int)*cp)) {
            return (1 + cp - name);
            /*skAppPrintErr("sensor name cannot contain whitespace characters");
             *return 1;
             */
        }
        if (('/' == *cp) || ('_' == *cp)) {
            return (1 + cp - name);
            /*skAppPrintErr("sensor name contains illegal character '%c'",
             *              *cp);
             *return 1;
             */
        }
    }

    /* check that begins with a letter */
    if ( !isalpha((int)*name)) {
        /* skAppPrintErr("sensors must begin with a letter"); */
        return -1;
    }

    return 0;
}


static void sksiteSensorPutInClass(sensorID_t sid, uint8_t class_id)
{
    sensor_struct_t *si = _siteGetSensorInfo(sid);

    if (class_id >= _siteGetClasssymCount() || si == NULL) {
        skAppPrintErr(("Cannot add sensor '%u' to class '%u'.\n"
                       "\tFix your site header and recompile.  Abort!"),
                      sid, class_id);
        assert(class_id < _siteGetClasssymCount());
        assert(_siteGetSensorInfo(sid) != NULL);
        abort();
    }

    if ( !(si->s_class_map & (1 << class_id))) {
        si->s_class_map |= (1 << class_id);
        ++si->s_class_count;
    }
}


int sksiteSensorGetClasses(
    uint32_t           *out_class_bitmap,
    sensorID_t          id)
{
    sensor_struct_t *si;

    si = _siteGetSensorInfo(id);
    if (si == NULL) {
        return -1;
    }
    if (out_class_bitmap) {
        *out_class_bitmap = si->s_class_map;
    }
    return (int)si->s_class_count;
}


int sksiteCreateTemporarySensor(
    const char         *sensor_name,
    const sk_vector_t  *class_names)
{
    size_t i, c;
    uint32_t sensor_count;
    char *cl_name;
    char *s_name;
    sensorID_t sid;

    /* make sure we have input */
    if (sensor_name == NULL
        || class_names == NULL
        || skVectorGetElementSize(class_names) != sizeof(char*))
    {
        return -1;
    }

    /* verify sensor name is of correct form and that a sensor of that
     * name does not already exist */
    if (sksiteSensorNameIsLegal(sensor_name)) {
        return -2;
    }
    if (_siteFindSensorName(sensor_name) != SK_INVALID_SENSOR) {
        return -3;
    }

    /* check count of classes */
    c = skVectorGetCount(class_names);
    if (c == 0 || c > MAX_CLASSES_PER_SENSOR) {
        return -4;
    }

    /* verify that all class names are valid */
    for (i = 0; i < c; ++i) {
        skVectorGetValue(&cl_name, class_names, i);
        if (_siteFindClassSym(cl_name) == SK_INVALID_CLASS) {
            return 1+i;
        }
    }

    /* input looks good. create the sensor */

    /* get the ID of this new sensor---this works since sensors are
     * currently created sequentially */
    sensor_count = _siteGetSensorCount();

    /* create a copy of the sensor_name */
    s_name = strdup(sensor_name);
    if (s_name == NULL) {
        return -1;
    }

    sid = sksiteAddSensor(s_name, sensor_count);
    for (i = 0; i < c; ++i) {
        skVectorGetValue(&cl_name, class_names, i);
        sksiteSensorPutInClass(sid, _siteFindClassSym(cl_name));
    }

    return 0;
}


/*
 *  match_count = sksiteParseSensorList(out_sensors, sensor_list);
 *
 *    Treats 'sensor_list' as a comma separated list of tokens
 *    representing the NAMES of sensors---does not accept sensor IDs.
 *    Appends an element to the 'out_sensors' vector, where the
 *    element holds the sensorID_t.
 *
 *    Returns the number of matches found, 0 for no matches, or -1 for
 *    memory error, NULL input, or if the vector's element's size is
 *    not sizeof(sensorID_t).
 */
int sksiteParseSensorList(
    sk_vector_t        *out_sensors,
    const char         *string)
{
    char *str_copy = NULL;
    char *cp;
    char *ep;
    sensorID_t id;
    int matches = 0;
    int rv = -1;

    /* check input */
    if (string == NULL) {
        return -1;
    }
    if (out_sensors != NULL) {
        if (skVectorGetElementSize(out_sensors) != sizeof(sensorID_t)) {
            return -1;
        }
    }

    /* make a copy of the input we can modify */
    str_copy = strdup(string);
    if (!str_copy) {
        goto END;
    }

    /* go over the list, converting ',' to '\0', then looking for a
     * sensor with the given name */
    cp = str_copy;
    while (*cp) {
        ep = strchr(cp, ',');
        if (ep == cp) {
            /* double comma, ignore */
            ++cp;
            continue;
        }
        if (ep != NULL) {
            *ep = '\0';
            ++ep;
        } else {
            /* force ep onto final '\0' */
            ep = cp + strlen(cp);
        }

        /* do we know this sensor? */
        id = _siteFindSensorName(cp);
        if (id == SK_INVALID_SENSOR) {
            goto END;
        }

        /* add to vector */
        ++matches;
        if (out_sensors != NULL) {
            if (0 != skVectorAppendValue(out_sensors, &id)) {
                goto END;
            }
        }

        cp = ep;
    }

    /* success */
    rv = matches;

  END:
    if (str_copy) {
        free(str_copy);
    }

    return rv;
}



/*
 *  *****  FILE OUTPUT FORMATS  *****
 */



static struct _fileformat_list {
    fileformat_struct_t    *fl_list;
    uint8_t                 fl_count;
} fileformats;

#ifdef TEST_SITE

static uint8_t _siteGetFileformatCount(void)
{
    return fileformats.fl_count;
}

static const char *_siteFileformatGetNameFast(fileFormat_t ffid)
{
    return fileformats.fl_list[(ffid)].ff_name;
}

static const char *_siteFileformatGetName(fileFormat_t ffid)
{
    return (((ffid) >= _siteGetFileformatCount())
            ? NULL
            : _siteFileformatGetNameFast(ffid));
}

#else /* TEST_SITE */

#define _siteGetFileformatCount()               \
    (fileformats.fl_count)

#define _siteFileformatGetNameFast(ffid)        \
    (fileformats.fl_list[(ffid)].ff_name)

#define _siteFileformatGetName(ffid)            \
    (((ffid) >= _siteGetFileformatCount())      \
     ? NULL                                     \
     : _siteFileformatGetNameFast(ffid))

#endif /* TEST_SITE */


static int _siteInitializeFileformats(size_t sz)
{
    if (sz > UINT8_MAX) {
        return -1;
    }
    fileformats.fl_list = calloc(1+sz, sizeof(fileformat_struct_t));
    fileformats.fl_count = (uint8_t)sz;
    if (fileformats.fl_list == NULL) {
        return -1;
    }
    return 0;
}


static fileFormat_t _siteFileformatAdd(
   const char          *name,
   uint8_t              id)
{
    fileformat_struct_t *ffi;

    /* check length of file format */
    if (strlen(name) > SK_MAX_STRLEN_FILE_FORMAT) {
        skAppPrintErr(("File format name '%s' is longer than allowed (%u)\n"
                       "\tFix you site header and recompile.  Abort!"),
                      name, SK_MAX_STRLEN_FILE_FORMAT);
        assert(strlen(name) <= SK_MAX_STRLEN_FILE_FORMAT);
        abort();
    }

    /* check id is o.k. */
    if (id >= _siteGetFileformatCount()) {
        skAppPrintErr(("File format id '%u' is larger than allowed (%u)\n"
                       "\tFix you site header and recompile.  Abort!"),
                      id, _siteGetFileformatCount());
        assert(id < _siteGetFileformatCount());
        abort();
    }

    ffi = &fileformats.fl_list[id];
    ffi->ff_id = (fileFormat_t)id;
    ffi->ff_name = name;

    return ffi->ff_id;
}


char *sksiteFileformatGetName(
    char           *out_buffer,
    fileFormat_t    id,
    size_t          bufsize)
{
    /* verify input */
    if (out_buffer == NULL || bufsize == 0) {
        return NULL;
    }

    /* get the name */
    if (NULL == _siteFileformatGetName(id)) {
        return NULL;
    }

    strncpy(out_buffer, _siteFileformatGetNameFast(id), bufsize);
    out_buffer[bufsize-1] = '\0';
    return out_buffer;
}


char *sksiteFileformatGetNameSafe(
    char           *out_buffer,
    fileFormat_t    id,
    size_t          bufsize)
{
    static const char unkn[] = "UNKN?";
    char *p = sksiteFileformatGetName(out_buffer, id, bufsize);
    if (p) {
        return p;
    }

    if (out_buffer && bufsize > 1+sizeof(unkn)) {
        strncpy(out_buffer, unkn, 1+sizeof(unkn));
        return out_buffer;
    }
    return out_buffer;
}


int sksiteFileformatIsValid(
    fileFormat_t    id)
{
    return (id < _siteGetFileformatCount());
}


#ifdef TEST_SITE
#  define fileformats
#endif



/*
 *  *****  COMPRESSION METHODS  *****
 */


static struct _compmethod_list {
    compmethod_struct_t    *cml_list;
    sk_compmethod_t         cml_count;
} compmethods;


#ifdef TEST_SITE

static uint8_t _siteGetCompmethodCount()
{
    return compmethods.cml_count;
}

static const char *_siteCompmethodGetNameFast(sk_compmethod_t cmid)
{
    return compmethods.cml_list[cmid].cm_name;
}

static const char *_siteCompmethodGetName(sk_compmethod_t cmid)
{
    return (((cmid) >= _siteGetCompmethodCount())
            ? NULL
            : _siteCompmethodGetNameFast(cmid));
}

#else /* TEST_SITE */

#define _siteGetCompmethodCount()               \
    (compmethods.cml_count)

#define _siteCompmethodGetNameFast(cmid)        \
    (compmethods.cml_list[(cmid)].cm_name)

#define _siteCompmethodGetName(cmid)            \
    (((cmid) >= _siteGetCompmethodCount())      \
     ? NULL                                     \
     : _siteCompmethodGetNameFast(cmid))

#endif /* TEST_SITE */


static int _siteInitializeCompmethods(size_t sz)
{
    if (sz > UINT8_MAX) {
        return -1;
    }
    compmethods.cml_list = calloc(1+sz, sizeof(compmethod_struct_t));
    compmethods.cml_count = (uint8_t)sz;
    if (compmethods.cml_list == NULL) {
        return -1;
    }
    return 0;
}


static sk_compmethod_t _siteCompmethodAdd(
    const char         *name,
    uint8_t             id)
{
    compmethod_struct_t *cmi;

#if 0
    /* check length of compression method name */
    if (strlen(name) > SK_MAX_STRLEN_FILE_FORMAT) {
        skAppPrintErr(("Compression method name '%s'"
                       " is longer than allowed (%u)\n"
                       "\tFix you site header and recompile.  Abort!"),
                      name, SK_MAX_STRLEN_FILE_FORMAT);
        assert(strlen(cmi->cm_name) <= SK_MAX_STRLEN_FILE_FORMAT);
        abort();
    }
#endif

    /* check id is o.k. */
    if (id >= _siteGetCompmethodCount()) {
        skAppPrintErr(("Compression method id '%u'"
                       " is larger than allowed (%u)\n"
                       "\tFix you site header and recompile.  Abort!"),
                      id, _siteGetCompmethodCount());
        assert(id < _siteGetCompmethodCount());
        abort();
    }

    cmi = &compmethods.cml_list[id];
    cmi->cm_id = (sk_compmethod_t)id;
    cmi->cm_name = name;

    return cmi->cm_id;
}


char *sksiteCompmethodGetName(
    char               *out_buffer,
    sk_compmethod_t     id,
    size_t              bufsize)
{
    /* verify input */
    if (out_buffer == NULL || bufsize == 0) {
        return NULL;
    }

    /* get the name */
    if (NULL == _siteCompmethodGetName(id)) {
        return NULL;
    }

    strncpy(out_buffer, _siteCompmethodGetNameFast(id), bufsize);
    out_buffer[bufsize-1] = '\0';
    return out_buffer;
}


int sksiteCompmethodIsAvailable(
    sk_compmethod_t     id)
{
    switch (id) {
      case SK_COMPMETHOD_NONE:
#ifdef HAVE_ZLIB_H
      case SK_COMPMETHOD_ZLIB:
#endif
#ifdef SK_LZO_HEADER_NAME
      case SK_COMPMETHOD_LZO1X:
#endif
        return 1;
    }

    return 0;
}


int sksiteCompmethodIsValid(
    sk_compmethod_t     id)
{
    return (((uint8_t)id) < _siteGetCompmethodCount());
}


sk_compmethod_t sksiteCompmethodGetBest(void)
{
#if   defined(SK_LZO_HEADER_NAME)
    return SK_COMPMETHOD_LZO1X;
#elif defined(HAVE_ZLIB_H)
    return SK_COMPMETHOD_ZLIB;
#else
    return SK_COMPMETHOD_NONE;
#endif
}


sk_compmethod_t sksiteCompmethodGetDefault(void)
{
    /* return the value from configure */
    return SK_ENABLE_OUTPUT_COMPRESSION;
}


// ==========================================================================


#define COMPMETHOD_STRING_BEST "best"

typedef enum {
    OPT_COMPRESSION_METHOD
} site_compmethod_opts_enum_t;

static struct option site_compmethod_opts[] = {
    {"compression-method",  REQUIRED_ARG, 0, OPT_COMPRESSION_METHOD},
    {0,0,0,0}               /* sentinel entry */
};


static int _siteCompmethodOptionsHandler(
    clientData          cData,
    int                 opt_index,
    char               *opt_arg)
{
    sk_compmethod_t *compression_method = (sk_compmethod_t*)cData;
    sk_stringmap_t *str_map = NULL;
    sk_stringmap_status_t rv_map;
    sk_stringmap_entry_t *map_entry;
    sk_compmethod_t cm;
    int rv = 1;

    assert(opt_index == OPT_COMPRESSION_METHOD);
    if (opt_index != OPT_COMPRESSION_METHOD) {
        abort();
    }

    /* create a stringmap of the available entries */
    if (SKSTRINGMAP_OK != skStringMapCreate(&str_map)) {
        skAppPrintErr("Unable to create stringmap");
        goto END;
    }
    if (skStringMapAddID(str_map, COMPMETHOD_STRING_BEST,
                         (sk_stringmap_id_t)sksiteCompmethodGetBest())
        != SKSTRINGMAP_OK)
    {
        goto END;
    }
    for (cm = 0; cm < _siteGetCompmethodCount(); ++cm) {
        if ( !sksiteCompmethodIsAvailable(cm)) {
            continue;
        }
        if (skStringMapAddID(str_map, _siteCompmethodGetNameFast(cm),
                             (sk_stringmap_id_t)cm) != SKSTRINGMAP_OK)
        {
            goto END;
        }
    }

    /* attempt to match */
    rv_map = skStringMapGetEntry(&map_entry, str_map, opt_arg);
    switch (rv_map) {
      case SKSTRINGMAP_OK:
        *compression_method = (uint8_t)map_entry->id;
        rv = 0;
        break;

      case SKSTRINGMAP_PARSE_AMBIGUOUS:
        skAppPrintErr("%s value '%s' is ambiguous",
                      site_compmethod_opts[OPT_COMPRESSION_METHOD].name,
                      opt_arg);
        goto END;

      case SKSTRINGMAP_PARSE_NO_MATCH:
        skAppPrintErr("%s value '%s' does not match any known method",
                      site_compmethod_opts[OPT_COMPRESSION_METHOD].name,
                      opt_arg);
        goto END;

      default:
        skAppPrintErr("Unexpected return value from string-map parser (%d)",
                      rv_map);
        goto END;
    }

  END:
    if (str_map) {
        skStringMapDestroy(str_map);
    }
    return rv;
}


int sksiteCompmethodOptionsRegister(
    sk_compmethod_t    *compression_method)
{
    *compression_method = sksiteCompmethodGetDefault();

    return optionsRegister(site_compmethod_opts,
                           &_siteCompmethodOptionsHandler,
                           compression_method);
}


void sksiteCompmethodOptionsUsage(
    FILE               *fh)
{
    int i;
    sk_compmethod_t cm;

    for (i = 0; site_compmethod_opts[i].name; ++i) {
        fprintf(fh, "--%s %s. ", site_compmethod_opts[i].name,
                SK_OPTION_HAS_ARG(site_compmethod_opts[i]));
        switch ((site_compmethod_opts_enum_t)(site_compmethod_opts[i].val)) {
          case OPT_COMPRESSION_METHOD:
            fprintf(fh, ("Set compression for binary output file(s).\n"
                         "\tDef. %s. Choices: %s [=%s]"),
                    _siteCompmethodGetNameFast(sksiteCompmethodGetDefault()),
                    COMPMETHOD_STRING_BEST,
                    _siteCompmethodGetNameFast(sksiteCompmethodGetBest()));
            for (cm = 0; cm < _siteGetCompmethodCount(); ++cm) {
                if ( !sksiteCompmethodIsAvailable(cm)) {
                    continue;
                }
                fprintf(fh, ", %s", _siteCompmethodGetNameFast(cm));
            }
            break;
        }
        fprintf(fh, "\n");
    }
}


// ===========================================================================

#ifdef TEST_SITE
#  define compmethods
#endif



/*
 *  ****  FLOW TYPES  *****
 */


static flowtypeID_t flowtype_count = 0;

static flowtype_struct_t *flowtypes = NULL;

#define _siteGetFlowtypeCount()                 \
    (flowtype_count)

#define _siteGetFlowtypeInfoFast(ftid)          \
    (&flowtypes[(ftid)])

#define _siteGetFlowtypeInfo(ftid)              \
    (((ftid) >= _siteGetFlowtypeCount())        \
     ? NULL                                     \
     : _siteGetFlowtypeInfoFast(ftid))

#define _siteFlowtypeInfoGetClassId(fti)        \
    ((fti)->ft_class)

#define _siteFlowtypeInfoGetClassName(fti)      \
    (_siteClasssymGetNameFast((fti)->ft_class))

#define _siteFlowtypeInfoGetClassNameLength(fti)        \
    (_siteClasssymGetStrlenFast((fti)->ft_class))

#define _siteFlowtypeInfoGetTypeId(fti)         \
    ((fti)->ft_type)

#define _siteFlowtypeInfoGetTypeName(fti)       \
    (_siteTypesymGetNameFast((fti)->ft_type))

#define _siteFlowtypeInfoGetTypeNameLength(fti) \
    (_siteTypesymGetStrlenFast((fti)->ft_type))

#define _siteFlowtypeInfoGetName(fti)           \
    ((fti)->ft_name)

#define _siteFlowtypeInfoGetPathPrefix(fti)     \
    ((fti)->ft_rel_dir)



/* Given a flowtype's name, return its Id, or SK_INVALID_FLOWTYPE */
static flowtypeID_t _siteFlowtypeNameToId(const char *name)
{
    flowtypeID_t i;

    for (i = 0; i < _siteGetFlowtypeCount(); ++i) {
        if (strcmp(_siteFlowtypeInfoGetName(_siteGetFlowtypeInfoFast(i)), name)
            == 0)
        {
            return i;
        }
    }

    return SK_INVALID_FLOWTYPE;
}



#if 0
/*
 *  count = _siteParseWordList(output_array, input_string, output_array_len);
 *
 *    Treats 'input_string' as a list of comma-separated tokens, where
 *    double comma is treated as a single comma.  Parses this list
 *    into an array whose length is at least 1, and puts pointers to
 *    each word into the caller-allocated array 'output_array' whose
 *    length is given by the 'output_array_len' parameter.
 *
 *    The function returns the number of words, or -1 if too more than
 *    'output_array_len' words were found.  This function will
 *    destructively modify input_string; the caller should pass in a
 *    mutable copy.  The values in output_array will point into
 *    input_string; this function allocates no memory.
 */
static int parseWordList(char **word_list, char *string, int max_word_list)
{
    int word_count = 0;
    char *cp;
    char *ep;

    cp = string;
    while (*cp) {
        ep = strchr(cp, ',');
        if (ep == cp) {
            /* double comma, ignore */
            ++cp;
            continue;
        }
        if (ep != NULL) {
            *ep = '\0';
            ++ep;
        } else {
            /* force ep onto final '\0' */
            ep = cp + strlen(cp);
        }

        if (word_count >= max_word_list) {
            return -1;
        }

        word_list[word_count++] = cp;
        cp = ep;
    }

    return word_count;
}
#endif /* 0 */




char *sksiteGetRootDir(
    char           *buffer,
    size_t          bufsize)
{
    if (bufsize < 1+strlen(data_rootdir)) {
        return NULL;
    }
    strncpy(buffer, data_rootdir, bufsize);
    return buffer;
}


int sksiteSetRootDir(
    const char     *rootdir)
{
    if (rootdir == NULL || rootdir[0] == '\0')  {
        return -1;
    }

    if (strlen(rootdir)+1 > sizeof(data_rootdir)) {
        return -1;
    }
    strncpy(data_rootdir, rootdir, sizeof(data_rootdir));
    return 0;
}


char *sksiteGeneratePathname(
    char           *buffer,
    size_t          bufsize,
    flowtypeID_t    flowtype,
    sensorID_t      sensor,
    uint32_t        timestamp,
    const char     *suffix,
    char          **reldir_begin,
    char          **filename_begin)
{
    flowtype_struct_t *fti;
    sensor_struct_t *si;
    /* convert to time_t for 64bit platforms */
    const time_t tt = (time_t)timestamp;
    struct tm trec;
    const char *path_prefix;
    const char *file_prefix;
    const char *sensor_name;
    int len;
    const char *suf = NULL;

    if (buffer == NULL || bufsize == 0) {
        return NULL;
    }

    fti = _siteGetFlowtypeInfo(flowtype);
    if (fti == NULL) {
        return NULL;
    }
    si = _siteGetSensorInfo(sensor);
    if (si == NULL) {
        return NULL;
    }

    /* set 'suf' to the suffix if it was provided and not the empty
     * string; ignore the leading '.' if suffix, it is added later. */
    if (suffix && *suffix) {
        suf = suffix;
        if (*suf == '.') {
            ++suf;
        }
    }

    path_prefix = _siteFlowtypeInfoGetPathPrefix(fti);
    file_prefix = _siteFlowtypeInfoGetName(fti);
    sensor_name = _siteSensorInfoGetName(si);

    gmtime_r(&tt, &trec);

    len = snprintf(buffer, bufsize,
                   "%s%s%s/%s/%04d/%02d/%02d/%s-%s_%04d%02d%02d.%02d%s%s",
                   data_rootdir,
#ifdef SK_ENABLE_SENSOR_IN_DATAPATH
                   "/", sensor_name,
#else
                   "", "",
#endif
                   path_prefix,
                   trec.tm_year + 1900, trec.tm_mon + 1, trec.tm_mday,
                   file_prefix, sensor_name,
                   trec.tm_year + 1900, trec.tm_mon + 1, trec.tm_mday,
                   trec.tm_hour,
                   ((suf == NULL) ? "" : "."),
                   ((suf == NULL) ? "" : suf));
    buffer[bufsize-1] = '\0';
    if (bufsize <= (size_t)len) {
        /* too short */
        return NULL;
    }

    if (reldir_begin) {
        *reldir_begin = &buffer[1+strlen(data_rootdir)];
    }
    if (filename_begin) {
        *filename_begin = strrchr(buffer, '/') + 1;
    }

    return buffer;
}


flowtypeID_t sksiteParseFilename(
    flowtypeID_t   *out_flowtype,
    sensorID_t     *out_sensor,
    uint32_t       *out_timestamp,
    char          **out_suffix,
    const char     *filename)
{
    char buf[PATH_MAX];
    const char *sp;
    char *ep;
    flowtypeID_t ft;
    unsigned long temp1, temp2;

    /* check input */
    if (!filename) {
        return SK_INVALID_FLOWTYPE;
    }

    /* copy file portion of filename into buf */
    sp = baseName_r(buf, filename, sizeof(buf));
    if (sp == NULL) {
        /* input name too long */
        return SK_INVALID_FLOWTYPE;
    }

    /* find the end of the prefix, e.g., "in-" */
    ep = strchr(sp, '-');
    if (NULL == ep) {
        return SK_INVALID_FLOWTYPE;
    }
    *ep = '\0';
    ++ep;

    /* lookup file type */
    ft = _siteFlowtypeNameToId(sp);
    if (ft == SK_INVALID_FLOWTYPE) {
        return SK_INVALID_FLOWTYPE;
    }
    if (out_flowtype) {
        *out_flowtype = ft;
    }

    /* find the sensor/timestamp separator */
    sp = ep;
    ep = strchr(sp, '_');
    if (NULL == ep) {
        return SK_INVALID_FLOWTYPE;
    }
    *ep = '\0';
    ++ep;

    if (out_sensor) {
        *out_sensor = _siteFindSensorName(sp);
    }

    /* move to start of time; convert "YYYYMMDD." into a single
     * integer, then pull out each part */
    sp = ep;
    errno = 0;
    temp1 = strtoul(sp, &ep, 10);
    if (sp == ep || *ep != '.' || (temp1 == ULONG_MAX && errno == ERANGE)
        || (temp1 < 19700101 || temp1 >= 20380119))
    {
        return SK_INVALID_FLOWTYPE;
    }

    /* now handle the hour "HH." or "HH\0" */
    sp = ep + 1;
    errno = 0;
    temp2 = strtoul(sp, &ep, 10);
    if (sp == ep || (*ep != '.' && *ep != '\0')
        || (temp2 == ULONG_MAX && errno == ERANGE) || (temp2 > 23))
    {
        return SK_INVALID_FLOWTYPE;
    }

    if (out_timestamp) {
        struct tm trec;
        time_t t;

        memset(&trec, 0, sizeof(struct tm));
        trec.tm_mday = temp1 % 100;
        temp1 /= 100;
        trec.tm_mon  = temp1 % 100 - 1;
        trec.tm_year = (temp1 / 100) - 1900;
        trec.tm_hour = temp2;
        t = timegm(&trec);
        if (t == (time_t)(-1)) {
            return SK_INVALID_FLOWTYPE;
        }
        *out_timestamp = (uint32_t)t;
    }

    if (out_suffix) {
        *out_suffix = ep;
    }

    return ft;
}


char *sksiteParseGeneratePath(
    char           *buffer,
    size_t          bufsize,
    const char     *filename,
    const char     *suffix,
    char          **reldir_begin,
    char          **filename_begin)
{
    flowtypeID_t flowtype;
    sensorID_t sensor;
    uint32_t timestamp;
    char *old_suffix;
    char new_suffix[PATH_MAX];

    if (sksiteParseFilename(&flowtype, &sensor, &timestamp, &old_suffix,
                            filename)
        == SK_INVALID_FLOWTYPE)
    {
        return NULL;
    }

    if (*old_suffix != '\0' && suffix == NULL) {
        /* there was a suffix on 'filename' and the caller didn't
         * provide a new suffix; append old suffix to new name */
        strncpy(new_suffix, old_suffix, sizeof(new_suffix));
        if (new_suffix[sizeof(new_suffix)-1] != '\0') {
            /* suffix too long */
            return NULL;
        }
        suffix = new_suffix;
    }

    return sksiteGeneratePathname(buffer, bufsize, flowtype, sensor, timestamp,
                                  suffix, reldir_begin, filename_begin);
}


flowtypeID_t sksiteLookupFlowtype(
    const char         *class,
    const char         *type)
{
    flowtype_struct_t *fti;
    flowtypeID_t i;
    uint8_t class_id;
    uint8_t type_id;

    if (class == NULL || type == NULL) {
        return SK_INVALID_FLOWTYPE;
    }

    class_id = _siteFindClassSym(class);
    type_id = _siteFindTypeSym(type);

    if (class_id == SK_INVALID_CLASS || type_id == SK_INVALID_TYPE) {
        return SK_INVALID_FLOWTYPE;
    }

    for (i = 0; i < _siteGetFlowtypeCount(); ++i) {
        fti = _siteGetFlowtypeInfoFast(i);
        if ((class_id == _siteFlowtypeInfoGetClassId(fti)) &&
            (type_id == _siteFlowtypeInfoGetTypeId(fti)))
        {
            return i;
        }
    }

    /* not found */
    return SK_INVALID_FLOWTYPE;
}


struct _flowtype_iter {
    uint8_t is_used[256];
    uint8_t current;
    uint8_t first;
    uint8_t last;
};


int sksiteSelectFlowtypesByClassType(
    flowtype_iter_t    *iter,
    const char         *class,
    const char         *type)
{
#define WORD_ARRAY_SIZE 256
    flowtype_struct_t *fti;
    flowtypeID_t i;
    int num_classes = 0;
    int num_types = 0;
    uint8_t class_id[UINT8_MAX];
    uint8_t type_id[UINT8_MAX];
    int rv = -1;

    /* initialize iterator */
    memset(iter, 0, sizeof(flowtype_iter_t));
    iter->first = UINT8_MAX;

    if (class) {
        /* get the symbol IDs for the class */
        num_classes = _siteParseClassList(class_id, class);
        if (num_classes <= 0) {
            /* bad class list */
            goto END;
        }
    }

    if (type) {
        /* get the symbol IDs for the type */
        num_types = _siteParseTypeList(type_id, type);
        if (num_types <= 0) {
            /* bad type list */
            goto END;
        }
    }

    /* loop over all the flowtypes */
    for (i = 0; i < _siteGetFlowtypeCount(); ++i) {
        fti = _siteGetFlowtypeInfoFast(i);

        if (num_classes == 0) {
            /* when no classes given, ignore all classes except
             * default */
            if ( !fti->ft_default_class) {
                continue;
            }
        } else if ( !class_id[_siteFlowtypeInfoGetClassId(fti)]) {
            /* flowtype's class not in user's list */
            continue;
        }

        /* We know class matches, now try the type */

        if (num_types == 0) {
            if ( !fti->ft_default_type_for_class) {
                /* when no types given, ignore all types except those
                 * that are default for a class */
                continue;
            }
        } else if ( !type_id[_siteFlowtypeInfoGetTypeId(fti)]) {
            /* flowtype's type not in user's list */
            continue;
        }

        /* if we make it here, we have a valid class/type pair */
        iter->is_used[i] = 1;

        /* set first and last */
        if (iter->first > iter->last) {
            iter->first = i;
            /* found at least one class/type pair */
            rv = 0;
        }
        iter->last = i;
    }

  END:
    return rv;
}



//flowtype_t sksiteFlowtypeIdGetByName(const char *ft_name)
//{
//    char buffer[64];
//    size_t len;
//    const char *cp;
//
//    /* find end of prefix, e.g., "in-" */
//    cp = strchr(ft_name, '-');
//    if (NULL == cp) {
//        return _siteFlowtypeNameToId(ft_name);
//    }
//
//    len = cp - ft_name;
//    if (len >= sizeof(buffer)) {
//        return SK_INVALID_FLOWTYPE;
//    } else {
//        strncpy(buffer, ft_name, len);
//        buffer[len] = '\0';
//    }
//
//    return _siteFlowtypeNameToId(buffer);
//}


//size_t sksiteFlowtypeIdGetName(
//    char       *out_ft_name,
//    size_t      name_len,
//    flowtype_t  id)
//{
//    fileTypeInfo *fti;
//    size_t len;
//
//    fti = _siteGetFlowtypeInfo(id);
//    if (fti == NULL) {
//        return 0;
//    }
//
//    len = strlen(_siteFlowtypeInfoGetName(fti));
//    if (out_ft_name != NULL && name_len > 0) {
//        strncpy(out_ft_name, _siteFlowtypeInfoGetName(fti), name_len);
//        out_ft_name[name_len-1] = '\0';
//    }
//
//    return len;
//}


/* Put the class of the flowtype into 'out_buffer' and return pointer
 * to out_buffer, or NULL on error. */
char *sksiteFlowtypeGetClass(
    char           *out_buffer,
    flowtypeID_t    ftid,
    size_t          bufsize)
{
    flowtype_struct_t *fti;

    if (out_buffer == NULL) {
        return NULL;
    }
    fti = _siteGetFlowtypeInfo(ftid);
    if (fti == NULL) {
        return NULL;
    }
    if (bufsize <= _siteFlowtypeInfoGetClassNameLength(fti)) {
        return NULL;
    }

    strncpy(out_buffer, _siteFlowtypeInfoGetClassName(fti), bufsize);
    out_buffer[bufsize-1] = '\0';
    return out_buffer;
}


char *sksiteFlowtypeGetClassSafe(
    char           *out_buffer,
    flowtypeID_t    ftid,
    size_t          bufsize)
{
    static const char unkn[] = "?";
    char *p = sksiteFlowtypeGetClass(out_buffer, ftid, bufsize);
    if (p) {
        return p;
    }

    if (out_buffer && bufsize > 1+sizeof(unkn)) {
        strncpy(out_buffer, unkn, 1+sizeof(unkn));
        return out_buffer;
    }
    return (char*)unkn;
}


/* Put the type of the flowtype into 'out_buffer', return pointer to
 * 'out_buffer' or NULL on error. */
char *sksiteFlowtypeGetType(
    char           *out_buffer,
    flowtypeID_t    ftid,
    size_t          bufsize)
{
    flowtype_struct_t *fti;

    if (out_buffer == NULL) {
        return NULL;
    }
    fti = _siteGetFlowtypeInfo(ftid);
    if (fti == NULL) {
        return NULL;
    }
    if (bufsize <= _siteFlowtypeInfoGetTypeNameLength(fti)) {
        return NULL;
    }

    strncpy(out_buffer, _siteFlowtypeInfoGetTypeName(fti), bufsize);
    out_buffer[bufsize-1] = '\0';
    return out_buffer;
}


char *sksiteFlowtypeGetTypeSafe(
    char           *out_buffer,
    flowtypeID_t    ftid,
    size_t          bufsize)
{
    static const char unkn[] = "?";
    char *p = sksiteFlowtypeGetType(out_buffer, ftid, bufsize);
    if (p) {
        return p;
    }

    if (out_buffer && bufsize > 1+sizeof(unkn)) {
        strncpy(out_buffer, unkn, 1+sizeof(unkn));
        return out_buffer;
    }
    return (char*)unkn;
}














int sksiteInitialize(int UNUSED(levels))
{
    static int initialized = 0;
    flowtype_struct_t *fti;
    uint8_t i;
    uint8_t j;
    size_t numFormats;
    size_t numCompr;

    if (initialized) {
        return 0;
    }
    initialized = 1;

    /* store the root_directory from configure */
    if (sksiteSetRootDir(SILK_DATA_ROOTDIR)) {
        skAppPrintErr("Data root directory is too long");
        abort();
    }

    /* compute number of file formats: FT_* macros; if the final entry
     * in the array is the empty string; remove it from format
     * count. */
    numFormats = (sizeof(fileOutputFormats)/sizeof(char*));
    if (fileOutputFormats[numFormats-1][0] == '\0') {
        --numFormats;
    }

    _siteInitializeFileformats(numFormats);

    /* loop over fileOutputFormats[] and create the file formats */
    for (i = 0; i < numFormats && fileOutputFormats[i][0]; ++i) {
        _siteFileformatAdd(fileOutputFormats[i], i);
    }
    if (_siteGetFileformatCount() != numFormats) {
        skAppPrintErr("Inconsistency in fileOutputFormats[] array.\n"
                      "\tFix your site header and recompile.  Abort!");
        assert(_siteGetFileformatCount() == numFormats);
        abort();
    }


    /* compute number of compression methods: SK_COMPMETHOD_* macros;
     * if the final entry in the array is the empty string; remove it
     * from method count. */
    numCompr = (sizeof(skCompressionMethods)/sizeof(char*));
    if (skCompressionMethods[numCompr-1][0] == '\0') {
        --numCompr;
    }

    _siteInitializeCompmethods(numCompr);

    /* loop over skCompressionMethods[] to create the compr methods */
    for (j = 0; j < numCompr && skCompressionMethods[j][0]; ++j) {
        _siteCompmethodAdd(skCompressionMethods[j], j);
    }
    if (_siteGetCompmethodCount() != numCompr) {
        skAppPrintErr("Inconsistency in skCompressionMethods[] array.\n"
                      "\tFix your site header and recompile.  Abort!");
        assert(_siteGetCompmethodCount() == numCompr);
        abort();
    }


    _siteInitializeClass(NUM_CLASSES);

    /* loop over the classInfo[] array to create class entries. */
    for (i = 0; i < NUM_CLASSES; ++i) {
        j = sksiteLookupAddClass(classInfo[i].className);
        if (j != i) {
            skAppPrintErr("Inconsistency in classInfo[] array.\n"
                          "\tFix your site header and recompile.  Abort!");
            assert(j == i);
            abort();
        }
    }
    if (_siteGetClasssymCount() != NUM_CLASSES) {
        skAppPrintErr("Inconsistency in classInfo[] array.\n"
                      "\tFix your site header and recompile.  Abort!");
        assert(_siteGetClasssymCount() == NUM_CLASSES);
        abort();
    }

    /* Create the type array */
    _siteInitializeType(numFileTypes);

    /* create flowtypes array */
    flowtypes = calloc(numFileTypes, sizeof(flowtype_struct_t));
    if (flowtypes == NULL) {
        skAppPrintErr("Out of memory!");
        abort();
    }

    /* loop over the outFInfo[] array to create flowtypes[] */
    for (i = 0; i < numFileTypes; ++i) {
        fti = &flowtypes[flowtype_count];
        fti->ft_id = i;
        fti->ft_name = outFInfo[i].filePrefix;
        fti->ft_rel_dir = outFInfo[i].pathPrefix;
        fti->ft_format = outFInfo[i].packType;
        fti->ft_version = SK_FILE_VERSION_ANY;

        if (strlen(fti->ft_name) > SK_MAX_STRLEN_FLOWTYPE) {
            skAppPrintErr(("Flowtype name '%s' is longer than allowed (%u).\n"
                           "\tFix your site header and recompile.  Abort!"),
                          fti->ft_name, SK_MAX_STRLEN_FLOWTYPE);
            assert(strlen(fti->ft_name) <= SK_MAX_STRLEN_FLOWTYPE);
            abort();
        }

        /* convert the class name to a number */
        j = sksiteLookupAddClass(outFInfo[i].className);
        if (j == NUM_CLASSES) {
            /* unknown class */
            skAppPrintErr(("Flowtype %d references unknown class %s.\n"
                           "\tFix your site header and recompile.  Abort!"),
                          fti->ft_id, outFInfo[i].className);
            assert(j != NUM_CLASSES);
            abort();
        }
        fti->ft_class = j;

        /* set the ft_default_class flag if appropriate */
        if (_siteFlowtypeInfoGetClassId(fti) == DEFAULT_CLASS) {
            fti->ft_default_class = 1;
        }

        /* lookup or create a new type */
        fti->ft_type = sksiteLookupAddType(outFInfo[i].dirPrefix);

        /* added this flowtype */
        ++flowtype_count;
    }
    if (flowtype_count != numFileTypes) {
        skAppPrintErr("Inconsistency in outFInfo[] array.\n"
                      "\tFix your site header and recompile.  Abort!");
        assert(flowtype_count == numFileTypes);
        abort();
    }

    /* loop over the classInfo[] array to verify consistency. */
    for (i = 0; i < NUM_CLASSES; ++i) {
        /* verify that data in classInfo squares with flowtypes */
        for (j = 0; j < classInfo[i].numTypes; ++j) {
            if (i != _siteFlowtypeInfoGetClassId(_siteGetFlowtypeInfoFast(classInfo[i].typeList[j])))
            {
                skAppPrintErr("Inconsistency in classInfo[] array.\n"
                              "\tFix your site header and recompile.  Abort!");
                assert(i == _siteFlowtypeInfoGetClassId(_siteGetFlowtypeInfoFast(classInfo[i].typeList[j])));
                abort();
            }
        }

        /* set default_type_for_class flag. */
        for (j = 0; j < classInfo[i].numDefaults; ++j) {
            fti =_siteGetFlowtypeInfoFast(classInfo[i].defaultList[j]);
            fti->ft_default_type_for_class = 1;
        }
    }


    _siteInitializeSensor(SENSOR_COUNT);

    /* loop over the sensorInfo[] array to create sensor_list[] */
    for (i = 0; sensorInfo[i].sensorName; ++i) {
        sksiteAddSensor(sensorInfo[i].sensorName, i);
        for (j = 0; j < sensorInfo[i].numClasses; ++j) {
            sksiteSensorPutInClass((sensorID_t)i,
                                   sensorInfo[i].classNumbers[j]);
        }
    }
    if ((_siteGetSensorCount() != numSensors)
        || (_siteGetSensorCount() != SENSOR_COUNT))
    {
        skAppPrintErr("Inconsistency in sensorInfo[] array.\n"
                      "\tFix your site header and recompile.  Abort!");
        assert(_siteGetSensorCount() == numSensors);
        assert(_siteGetSensorCount() == SENSOR_COUNT);
        abort();
    }

    return 0;
}
